from django import forms

from individuals.models import *
from variants.models import *

from diseases.models import Disease, HGMDPhenotype

from filter_analysis.models import * 
from genes.models import *

from django.forms import ModelForm

# from django_select2 import *

from django.core.exceptions import ValidationError

from django_select2.forms import (HeavySelect2MultipleWidget, HeavySelect2Widget, ModelSelect2MultipleWidget, ModelSelect2TagWidget, ModelSelect2Widget, Select2MultipleWidget, Select2Widget)

from django.db.models import Q

def validate_fail_always(value):
    raise ValidationError('%s not valid. Infact nothing is valid!' % value)


# class MultiOmimChoices(ModelSelect2MultipleWidget):
#     queryset = Disease
#     search_fields = ['name__icontains', ]

class MultiOmimChoices(ModelSelect2Widget):
    model = Disease
    search_fields = [
        'name__icontains'
    ]
    def label_from_instance(self, obj):
        return force_text(obj.name).upper()

# class MultiHgmdChoices(AutoModelSelect2MultipleField):
#     queryset = HGMDPhenotype.objects
#     search_fields = ['name__icontains', ]
# class MultiCgdChoices(AutoModelSelect2MultipleField):
#     queryset = CGDCondition.objects
#     search_fields = ['name__icontains', ]
# class MultiCgdManifestationChoices(AutoModelSelect2MultipleField):
#     queryset = Manifestation.objects
#     search_fields = ['name__icontains', ]



# Create the form class.
class Filter(ModelForm):
     class Meta:
         model = FilterAnalysis
         fields = '__all__'

class FamilyFilter(ModelForm):
    class Meta:
        model = FamilyFilterAnalysis
        fields = '__all__'


#GENE_GROUPS= [(x.id, x.name) for x in GeneGroup.objects.all()]
FIELDS = Variant()._meta.get_fields()

class FilterAnalysisForm(forms.Form):
#    snp_id = forms.CharField(max_length=50, required=False)
    chr = forms.CharField(max_length=50, required=False, label='CHR')
    pos = forms.CharField(max_length=50, required=False, label='POS')
    
    snp_list = forms.CharField(widget=forms.Textarea, required=False, label='SNP LIST')
    exclude_snp_list = forms.CharField(widget=forms.Textarea, required=False, label='EXCLUDE SNP LIST')
    
    #individuals = forms.MultipleChoiceField(choices=[(x.id, x.name) for x in Individual.objects.filter().order_by('id')], required=False)

    # NUMBER_CHOICES = ['1', '2', '3', '4']
    # number = forms.ChoiceField(widget=Select2Widget, choices=NUMBER_CHOICES, required=False)


    individuals = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=Individual.objects.all().order_by('id'),
        search_fields=['name__icontains'],
    ), queryset=Individual.objects.all().order_by('id'), required=True)

    # individuals = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False, label='INDIVIDUALS')

    exclude_individuals = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False, label='EXCLUDE INDIVIDUALS')

    groups = forms.ModelMultipleChoiceField(queryset=Group.objects.all().order_by('id'), required=False, label='GROUPS')
    exclude_groups = forms.ModelMultipleChoiceField(queryset=Group.objects.all().order_by('id'), required=False, label='EXCLUDE GROUPS')

    
    # CHOICES = (('', ''),
    #            ('recessive', 'recessive'),
    #            ('dominant', 'dominant'),
    #            )
    # inheritance_model = forms.ChoiceField(required=False, choices=CHOICES, label="Inheritance Model")
    CHOICES = (('', ''),
               ('homozygous', 'HOMOZYGOUS  (Ex. 1/1, 2/1, 1/2)'),
               ('heterozygous', 'HETEROZYGOUS  (Ex. 0/1, 0/2, 1/0, 2/0)'),
               )
    mutation_type = forms.ChoiceField(required=False, choices=CHOICES, label="MUTATION TYPE")
    
    gene_list = forms.CharField(widget=forms.Textarea, required=False, label="GENE LIST")
    exclude_gene_list = forms.CharField(widget=forms.Textarea, required=False, label="EXCLUDE GENE LIST")
#    genegroups = forms.ChoiceField(required=False, choices=GENE_GROUPS, label="Gene Groups")
    genegroups = forms.ModelChoiceField(queryset=GeneGroup.objects.all(), empty_label="-------------", required=False)
    
    genes_in_common = forms.BooleanField(required=False, label="SHOW ONLY VARIANTS PRESENT IN COMMON GENES BETWEEN ALL THE INDIVIDUALS SELECTED")
    positions_in_common = forms.BooleanField(required=False, label="SHOW ONLY VARIANTS AT EXACTLY SAME POSITION BETWEEN ALL THE INDIVIDUALS SELECTED")
    is_at_hgmd = forms.BooleanField(required=False, label="SHOW ONLY VARIANTS PRESENT AT HGMD")
    exclude_varisnp = forms.BooleanField(required=False, label="EXCLUDE VARIANTS AT VARISNP")
    
    

    #denormalize, should be a simple select or even in python :)
    CHOICES = [(x, x.replace('_', ' ')) for x in ['CDS', 'CHROMOSOME_LARGE DELETION', 'CODON_CHANGE', 'CODON_INSERTION', 'CODON_CHANGE_PLUS CODON_INSERTION', 'CODON_DELETION', 'CODON_CHANGE_PLUS CODON_DELETION', 'DOWNSTREAM', 'EXON', 'EXON_DELETED', 'FRAME_SHIFT', 'GENE', 'INTERGENIC', 'INTERGENIC_CONSERVED', 'INTRAGENIC', 'INTRON', 'INTRON_CONSERVED', 'MICRO_RNA', 'NON_SYNONYMOUS_CODING', 'NON_SYNONYMOUS_START', 'NON_SYNONYMOUS_STOP', 'RARE_AMINO_ACID', 'SPLICE_SITE_ACCEPTOR', 'SPLICE_SITE_DONOR', 'SPLICE_SITE_REGION', 'SPLICE_SITE_BRANCH', 'SPLICE_SITE_BRANCH_U12', 'STOP_LOST', 'START_GAINED', 'START_LOST', 'STOP_GAINED', 'SYNONYMOUS_CODING', 'SYNONYMOUS_START', 'SYNONYMOUS_STOP', 'TRANSCRIPT', 'REGULATION', 'UPSTREAM', 'UTR_3_PRIME', 'UTR_3_DELETED', 'UTR_5_PRIME', 'UTR_5_DELETED']]

    # use SnpeffAnnotation
    # (x[0], x[0].replace('_', ' ')) for x in Variant.objects.values_list('snpeff_effect').distinct().order_by('snpeff_effect')

    effect = forms.MultipleChoiceField(choices=CHOICES, required=False)
    # effect = forms.MultipleChoiceField(choices=[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_effect').distinct().order_by('snpeff_effect')], required=False)
    
    CHOICES = [(x, x.replace('_', ' ')) for x in ['NONE', 'SILENT', 'MISSENSE', 'NONSENSE']]

    #[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_functional_class').distinct().exclude(snpeff_functional_class='').order_by('snpeff_functional_class')]

    func_class = forms.MultipleChoiceField(choices=CHOICES, required=False)
    

    CHOICES = [(x, x.replace('_', ' ')) for x in ['HIGH', 'MODERATE', 'MODIFIER', 'LOW']]

    #[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_impact').distinct().order_by('snpeff_impact')]

    impact = forms.MultipleChoiceField(choices=CHOICES, required=False)

    CHOICES = (
               ('.', '.'),
               ('PASS', 'PASS'),
               )
    filter = forms.MultipleChoiceField(choices=CHOICES, required=False)

    # filter = forms.MultipleChoiceField(choices=[(x[0], x[0]) for x in Variant.objects.values_list('filter').distinct().order_by('filter')], required=False)
    
    dbsnp = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN LATEST DBSNP BUILD")
    
    #100genomes freqeuncy
    CHOICES = (('<', '<= '),
               ('>', '>= '),
               ('=', '='),)
    
    genomes1000 = forms.CharField(max_length=50, required=False, label="1000 GENOMES FREQUENCY")
    genomes1000_exclude = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN 1000GENOMES")
    
    #DBSNP Frequency
#    dbsnp_freq_option = forms.ChoiceField(required=False, choices=CHOICES, label="DBSNP Frequency")
    dbsnp_frequency = forms.CharField(max_length=50, required=False, label="")
    dbsnp_exclude = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN DBSNP")
    
    #Variation Server Frequency
#    variationserver_option = forms.ChoiceField(required=False, choices=CHOICES, label="ESP5400 Frequency")
    esp_frequency = forms.CharField(max_length=50, required=False, label="")
    esp_exclude = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN EXOME SEQUENCING PROJECT")
    
    # hi_frequency = forms.CharField(max_length=50, required=False, label="")
    # hi_exclude = forms.BooleanField(required=False, label="Exclude all variants not in genes with Haploinsufficiency")


    #SIFT
#    ljb_sift_class = forms.ChoiceField(choices=[(x[0], x[0]) for x in Variant.objects.values_list('ljb_sift_pred').distinct().exclude(ljb_sift_pred='').order_by('ljb_sift_pred')], required=False)
    
    sift = forms.CharField(max_length=50, required=False, label="SIFT SCORE")
    sift_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT SIFT SCORE")
    
    #POLYPHEN
    # polyphen_option = forms.ChoiceField(required=False, choices=CHOICES, label="Polyphen2 Option")
    polyphen = forms.CharField(max_length=50, required=False, label="POLYPHEN2 FREQUENCY")
    polyphen_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT POLYPHEN SCORE")

    cadd = forms.CharField(max_length=50, required=False, label="CADD SCORE")
    cadd_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT CADD SCORE")

    mcap = forms.CharField(max_length=50, required=False, label="M-CAP SCORE")
    mcap_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT M-CAP SCORE")

    rf_score = forms.CharField(max_length=50, required=False, label="RF SCORE")
    rf_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT RF SCORE")

    ada_score = forms.CharField(max_length=50, required=False, label="ADA SCORE")
    ada_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT ADA SCORE")
    
    mendelmd_score = forms.CharField(max_length=50, required=False, label="MENDEL,MD SCORE")
    
    dbsnp_option = forms.ChoiceField(required=False, choices=CHOICES, label="DBSNP BUILD")#initial='>',
    dbsnp_build = forms.CharField(max_length=50, required=False, label="DBSNP BUILD")#initial='130'
    
    read_depth_option = forms.ChoiceField(required=False, choices=CHOICES, label="READ DEPTH")
    read_depth = forms.CharField(max_length=50, required=False, label="")
    
    qual_option = forms.ChoiceField(required=False, choices=CHOICES, label="QUAL")
    qual = forms.CharField(max_length=50, required=False, label="")
    
    variants_per_gene_option = forms.ChoiceField(required=False, choices=CHOICES, label="VARIANTS PER GENE")
    variants_per_gene = forms.CharField(max_length=50, required=False, label="")
    
    #diseases
    # diseases = forms.MultipleChoiceField(choices=[(x[0], x[0]) for x in Disease.objects.values_list('name').distinct()], required=False)
    
    fields = forms.MultipleChoiceField(choices=FIELDS, required=False)
    
    # conditions = forms.ModelMultipleChoiceField(queryset=CGDCondition.objects.filter().order_by('name'), required=False)
    
    # omim = forms.ModelMultipleChoiceField(queryset=Disease.objects.filter().order_by('name'), required=False)
    
    # omim = MultiOmimChoices()

    omim = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=Disease.objects.all(),
        search_fields=['name__icontains'],
    ), queryset=Disease.objects.all(), required=True)

    cgd = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=CGDCondition.objects.all(),
        search_fields=['name__icontains'],
    ), queryset=CGDCondition.objects.all(), required=True)
    # hgmd = MultiHgmdChoices()

    cgdmanifestation = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=Manifestation.objects.all(),
        search_fields=['name__icontains'],
    ), queryset=Manifestation.objects.all(), required=True)
    
    # hgmd = forms.ModelMultipleChoiceField(queryset=HGMDPhenotype.objects.filter().order_by('name'), required=False)

    genelists = forms.ModelMultipleChoiceField(queryset=GeneList.objects.all().order_by('name'), required=False, label="SAVED GENE LIST")
    exclude_genelists = forms.ModelMultipleChoiceField(queryset=GeneList.objects.all().order_by('name'), required=False, label="EXCLUDE SAVED GENE LIST")

    def __init__(self, user=None, *args, **kwargs):

        super(FilterAnalysisForm, self).__init__(*args, **kwargs)

        if not user.is_authenticated:
            # print('user None', user)
            self.fields['individuals'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False, label='INDIVIDUALS')
            self.fields['exclude_individuals'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False, label='INDIVIDUALS')
        else:
            print('user', user)
            if user.is_superuser:
                self.fields['individuals'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter().order_by('id'), required=False, label='INDIVIDUALS')
                self.fields['exclude_individuals'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter().order_by('id'), required=False, label='INDIVIDUALS')
            else:
                self.fields['individuals'] = forms.ModelMultipleChoiceField(
                    queryset=Individual.objects.filter(Q(user=user) | Q(user=None)).order_by('id'), required=False, label='INDIVIDUALS')
                self.fields['exclude_individuals'] = forms.ModelMultipleChoiceField(
                    queryset=Individual.objects.filter(Q(user=user) | Q(user=None)).order_by('id'), required=False, label='INDIVIDUALS')
        # self.fields['sift'].widget.attrs['readonly'] = True
        # self.fields['polyphen'].widget.attrs['readonly'] = True
        # self.fields['genomes1000'].widget.attrs['readonly'] = True
        # self.fields['dbsnp_frequency'].widget.attrs['readonly'] = True
        # self.fields['esp_frequency'].widget.attrs['readonly'] = True
         

#trio analysis
class FamilyAnalysisForm(forms.Form):

    chr = forms.CharField(max_length=50, required=False, label='CHR')
    pos = forms.CharField(max_length=50, required=False, label='POS')
    
    snp_list = forms.CharField(widget=forms.Textarea, required=False, label='SNP LIST')
    exclude_snp_list = forms.CharField(widget=forms.Textarea, required=False, label='EXCLUDE SNP LIST')
    
    #individuals = forms.MultipleChoiceField(choices=[(x.id, x.name) for x in Individual.objects.filter().order_by('id')], required=False)
    individuals = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False, label='INDIVIDUALS')
    exclude_individuals = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False, label='EXCLUDE INDIVIDUALS')

    groups = forms.ModelMultipleChoiceField(queryset=Group.objects.all().order_by('id'), required=False, label='GROUPS')
    exclude_groups = forms.ModelMultipleChoiceField(queryset=Group.objects.all().order_by('id'), required=False, label='EXCLUDE GROUPS')

    
    # CHOICES = (('', ''),
    #            ('recessive', 'recessive'),
    #            ('dominant', 'dominant'),
    #            )
    # inheritance_model = forms.ChoiceField(required=False, choices=CHOICES, label="Inheritance Model")
    CHOICES = (('', ''),
               ('homozygous', 'HOMOZYGOUS  (Ex. 1/1, 2/1, 1/2)'),
               ('heterozygous', 'HETEROZYGOUS  (Ex. 0/1, 0/2, 1/0, 2/0)'),
               )
    mutation_type = forms.ChoiceField(required=False, choices=CHOICES, label="MUTATION TYPE")
    
    gene_list = forms.CharField(widget=forms.Textarea, required=False, label="GENE LIST")
    exclude_gene_list = forms.CharField(widget=forms.Textarea, required=False, label="EXCLUDE GENE LIST")
#    genegroups = forms.ChoiceField(required=False, choices=GENE_GROUPS, label="Gene Groups")
    genegroups = forms.ModelChoiceField(queryset=GeneGroup.objects.all(), empty_label="-------------", required=False)
    
    genes_in_common = forms.BooleanField(required=False, label="SHOW ONLY VARIANTS PRESENT IN COMMON GENES BETWEEN ALL THE INDIVIDUALS SELECTED")
    positions_in_common = forms.BooleanField(required=False, label="SHOW ONLY VARIANTS AT EXACTLY SAME POSITION BETWEEN ALL THE INDIVIDUALS SELECTED")
    is_at_hgmd = forms.BooleanField(required=False, label="SHOW ONLY VARIANTS PRESENT AT HGMD")
    exclude_varisnp = forms.BooleanField(required=False, label="EXCLUDE VARIANTS AT VARISNP")
    
    

    #denormalize, should be a simple select or even in python :)
    # CHOICES = [(x, x.replace('_', ' ')) for x in ['CDS', 'CHROMOSOME_LARGE DELETION', 'CODON_CHANGE', 'CODON_INSERTION', 'CODON_CHANGE_PLUS CODON_INSERTION', 'CODON_DELETION', 'CODON_CHANGE_PLUS CODON_DELETION', 'DOWNSTREAM', 'EXON', 'EXON_DELETED', 'FRAME_SHIFT', 'GENE', 'INTERGENIC', 'INTERGENIC_CONSERVED', 'INTRAGENIC', 'INTRON', 'INTRON_CONSERVED', 'MICRO_RNA', 'NON_SYNONYMOUS_CODING', 'NON_SYNONYMOUS_START', 'NON_SYNONYMOUS_STOP', 'RARE_AMINO_ACID', 'SPLICE_SITE_ACCEPTOR', 'SPLICE_SITE_DONOR', 'SPLICE_SITE_REGION', 'SPLICE_SITE_BRANCH', 'SPLICE_SITE_BRANCH_U12', 'STOP_LOST', 'START_GAINED', 'START_LOST', 'STOP_GAINED', 'SYNONYMOUS_CODING', 'SYNONYMOUS_START', 'SYNONYMOUS_STOP', 'TRANSCRIPT', 'REGULATION', 'UPSTREAM', 'UTR_3_PRIME', 'UTR_3_DELETED', 'UTR_5_PRIME', 'UTR_5_DELETED']]

    # use SnpeffAnnotation
    # (x[0], x[0].replace('_', ' ')) for x in Variant.objects.values_list('snpeff_effect').distinct().order_by('snpeff_effect')

    # effect = forms.MultipleChoiceField(choices=CHOICES, required=False)
    # effect = forms.MultipleChoiceField(choices=[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_effect').distinct().order_by('snpeff_effect')], required=False)
    CHOICES = [(x, x.replace('_', ' ')) for x in ['CDS', 'CHROMOSOME_LARGE DELETION', 'CODON_CHANGE', 'CODON_INSERTION', 'CODON_CHANGE_PLUS CODON_INSERTION', 'CODON_DELETION', 'CODON_CHANGE_PLUS CODON_DELETION', 'DOWNSTREAM', 'EXON', 'EXON_DELETED', 'FRAME_SHIFT', 'GENE', 'INTERGENIC', 'INTERGENIC_CONSERVED', 'INTRAGENIC', 'INTRON', 'INTRON_CONSERVED', 'MICRO_RNA', 'NON_SYNONYMOUS_CODING', 'NON_SYNONYMOUS_START', 'NON_SYNONYMOUS_STOP', 'RARE_AMINO_ACID', 'SPLICE_SITE_ACCEPTOR', 'SPLICE_SITE_DONOR', 'SPLICE_SITE_REGION', 'SPLICE_SITE_BRANCH', 'SPLICE_SITE_BRANCH_U12', 'STOP_LOST', 'START_GAINED', 'START_LOST', 'STOP_GAINED', 'SYNONYMOUS_CODING', 'SYNONYMOUS_START', 'SYNONYMOUS_STOP', 'TRANSCRIPT', 'REGULATION', 'UPSTREAM', 'UTR_3_PRIME', 'UTR_3_DELETED', 'UTR_5_PRIME', 'UTR_5_DELETED']]

    effect = forms.MultipleChoiceField(choices=CHOICES, required=False)

    CHOICES = [(x, x.replace('_', ' ')) for x in ['NONE', 'SILENT', 'MISSENSE', 'NONSENSE']]

    func_class = forms.MultipleChoiceField(choices=CHOICES, required=False)
    

    CHOICES = [(x, x.replace('_', ' ')) for x in ['HIGH', 'MODERATE', 'MODIFIER', 'LOW']]

    #[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_impact').distinct().order_by('snpeff_impact')]

    impact = forms.MultipleChoiceField(choices=CHOICES, required=False)
    
    # filter = forms.MultipleChoiceField(choices=[(x[0], x[0]) for x in Variant.objects.values_list('filter').distinct().order_by('filter')], required=False)
    CHOICES = (
               ('.', '.'),
               ('PASS', 'PASS'),
               )
    filter = forms.MultipleChoiceField(choices=CHOICES, required=False)

    dbsnp = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN LATEST DBSNP BUILD")
    
    #100genomes freqeuncy
    CHOICES = (('<', '<= '),
               ('>', '>= '),
               ('=', '='),)
    
    genomes1000 = forms.CharField(max_length=50, required=False, label="1000 GENOMES FREQUENCY")
    genomes1000_exclude = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN 1000GENOMES")
    
    #DBSNP Frequency
#    dbsnp_freq_option = forms.ChoiceField(required=False, choices=CHOICES, label="DBSNP Frequency")
    dbsnp_frequency = forms.CharField(max_length=50, required=False, label="")
    dbsnp_exclude = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN DBSNP")
    
    #Variation Server Frequency
#    variationserver_option = forms.ChoiceField(required=False, choices=CHOICES, label="ESP5400 Frequency")
    esp_frequency = forms.CharField(max_length=50, required=False, label="")
    esp_exclude = forms.BooleanField(required=False, label="EXCLUDE ALL VARIANTS PRESENT IN EXOME SEQUENCING PROJECT")
    
    # hi_frequency = forms.CharField(max_length=50, required=False, label="")
    # hi_exclude = forms.BooleanField(required=False, label="Exclude all variants not in genes with Haploinsufficiency")


    #SIFT
#    ljb_sift_class = forms.ChoiceField(choices=[(x[0], x[0]) for x in Variant.objects.values_list('ljb_sift_pred').distinct().exclude(ljb_sift_pred='').order_by('ljb_sift_pred')], required=False)
    
    sift = forms.CharField(max_length=50, required=False, label="SIFT SCORE")
    sift_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT SIFT SCORE")
    
    #POLYPHEN
    # polyphen_option = forms.ChoiceField(required=False, choices=CHOICES, label="Polyphen2 Option")
    polyphen = forms.CharField(max_length=50, required=False, label="POLYPHEN2 FREQUENCY")
    polyphen_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT POLYPHEN SCORE")

    cadd = forms.CharField(max_length=50, required=False, label="CADD SCORE")
    cadd_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT CADD SCORE")

    rf_score = forms.CharField(max_length=50, required=False, label="RF SCORE")
    rf_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT RF SCORE")

    ada_score = forms.CharField(max_length=50, required=False, label="ADA SCORE")
    ada_exclude = forms.BooleanField(required=False, label="EXCLUDE VARIANTS WITHOUT ADA SCORE")
    
    mendelmd_score = forms.CharField(max_length=50, required=False, label="MENDEL,MD SCORE")
    
    dbsnp_option = forms.ChoiceField(required=False, choices=CHOICES, label="DBSNP BUILD")#initial='>',
    dbsnp_build = forms.CharField(max_length=50, required=False, label="DBSNP BUILD")#initial='130'
    
    read_depth_option = forms.ChoiceField(required=False, choices=CHOICES, label="READ DEPTH")
    read_depth = forms.CharField(max_length=50, required=False, label="")
    
    qual_option = forms.ChoiceField(required=False, choices=CHOICES, label="QUAL")
    qual = forms.CharField(max_length=50, required=False, label="")
    
    variants_per_gene_option = forms.ChoiceField(required=False, choices=CHOICES, label="VARIANTS PER GENE")
    variants_per_gene = forms.CharField(max_length=50, required=False, label="")
    
    #diseases
    # diseases = forms.MultipleChoiceField(choices=[(x[0], x[0]) for x in Disease.objects.values_list('name').distinct()], required=False)
    
    fields = forms.MultipleChoiceField(choices=FIELDS, required=False)
    
    omim = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=Disease.objects.all(),
        search_fields=['name__icontains'],
    ), queryset=Disease.objects.all(), required=True)

    cgd = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=CGDCondition.objects.all(),
        search_fields=['name__icontains'],
    ), queryset=CGDCondition.objects.all(), required=True)
    # hgmd = MultiHgmdChoices()

    cgdmanifestation = forms.ModelMultipleChoiceField(widget=ModelSelect2MultipleWidget(
        queryset=Manifestation.objects.all(),
        search_fields=['name__icontains'],
    ), queryset=Manifestation.objects.all(), required=True)
    
    
    # hgmd = forms.ModelMultipleChoiceField(queryset=HGMDPhenotype.objects.filter().order_by('name'), required=False)

    genelists = forms.ModelMultipleChoiceField(queryset=GeneList.objects.all().order_by('name'), required=False, label="SAVED GENE LIST")
    exclude_genelists = forms.ModelMultipleChoiceField(queryset=GeneList.objects.all().order_by('name'), required=False, label="EXCLUDE SAVED GENE LIST")
    
    mother = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)
    father = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)
    children = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)
    INHERITANCE_CHOICES = (('', ''),
               ('recessive', 'Autosomal Recessive'),
               ('dominant', 'Autosomal Dominant'),
               ('compound heterozygous', 'Autosomal Compound Heterozygous'),
               ('xlinked recessive', 'X-linked Recessive'),
               ('xlinked dominant', 'X-linked Dominant'),               # ('recessive denovo', 'Autsomal Recessive denovo'),
               # ('dominant denovo', 'Autsomal Dominant denovo'),
               # ('xlinked dominant denovo', 'X-linked dominant de novo'),
               # ('xlinked recessive denovo', 'X-linked Recessive de novo'),

               )
    inheritance_option = forms.ChoiceField(choices=INHERITANCE_CHOICES, required=False)
    
    def __init__(self, user=None, *args, **kwargs):
        
        super(FamilyAnalysisForm, self).__init__(*args, **kwargs)

        if not user.is_authenticated:
            # print('user None', user)
            self.fields['mother'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False)
            self.fields['father'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False)
            self.fields['children'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False)
            self.fields['exclude_individuals'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False)
        else:
            # print('user', user)
            self.fields['mother'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=user).order_by('id'), required=False)
            self.fields['father'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=user).order_by('id'), required=False)
            self.fields['children'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=user).order_by('id'), required=False)
            self.fields['exclude_individuals'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=user).order_by('id'), required=False)


# INDIVIDUALS = [(x.id, x.name) for x in Individual.objects.all().order_by('id')]
INDIVIDUALS = []

#forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'))
class FilterWiZardForm1(forms.Form):
    individuals = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)#choices=INDIVIDUALS, 
    exclude_individuals = forms.MultipleChoiceField(choices=INDIVIDUALS, required=False)
    
    # groups = forms.MultipleChoiceField(choices=[(x.id, x.name) for x in Group.objects.all().order_by('id')], required=False)
    
    # exclude_groups = forms.MultipleChoiceField(choices=[(x.id, x.name) for x in Group.objects.all().order_by('id')], required=False)
    
    snp_list = forms.CharField(widget=forms.Textarea, required=False)
    exclude_snp_list = forms.CharField(widget=forms.Textarea, required=False)
    
    
    gene_list = forms.CharField(widget=forms.Textarea, required=False)
    exclude_gene_list = forms.CharField(widget=forms.Textarea, required=False)
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        INDIVIDUALS = [(x.id, x.name) for x in Individual.objects.all().order_by('id')]

    
class FilterWiZardForm2(forms.Form):
    #variant type
    CHOICES = (('', ''),
               ('homozygous', 'Homozygous  (Ex. 1/1, 2/1, 1/2)'),
               ('heterozygous', 'Heterozygous  (Ex. 0/1, 0/2, 1/0, 2/0)'),
               )
    mutation_type = forms.ChoiceField(required=False, choices=CHOICES, label="Mutation Type", initial="homozygous")

    #About Variant
    chr = forms.CharField(max_length=50, required=False)
    pos = forms.CharField(max_length=50, required=False)
    
    genegroups = forms.ModelChoiceField(queryset=GeneGroup.objects.all(), empty_label="-------------", required=False)
    
    
    CHOICES = ['CDS', 'CHROMOSOME_LARGE DELETION', 'CODON_CHANGE', 'CODON_INSERTION', 'CODON_CHANGE_PLUS CODON_INSERTION', 'CODON_DELETION', 'CODON_CHANGE_PLUS CODON_DELETION', 'DOWNSTREAM', 'EXON', 'EXON_DELETED', 'FRAME_SHIFT', 'GENE', 'INTERGENIC', 'INTERGENIC_CONSERVED', 'INTRAGENIC', 'INTRON', 'INTRON_CONSERVED', 'MICRO_RNA', 'NON_SYNONYMOUS_CODING', 'NON_SYNONYMOUS_START', 'NON_SYNONYMOUS_STOP', 'RARE_AMINO_ACID', 'SPLICE_SITE_ACCEPTOR', 'SPLICE_SITE_DONOR', 'SPLICE_SITE_REGION', 'SPLICE_SITE_BRANCH', 'SPLICE_SITE_BRANCH_U12', 'STOP_LOST', 'START_GAINED', 'START_LOST', 'STOP_GAINED', 'SYNONYMOUS_CODING', 'SYNONYMOUS_START', 'SYNONYMOUS_STOP', 'TRANSCRIPT', 'REGULATION', 'UPSTREAM', 'UTR_3_PRIME', 'UTR_3_DELETED', 'UTR_5_PRIME', 'UTR_5_DELETED']

    # use SnpeffAnnotation
    # (x[0], x[0].replace('_', ' ')) for x in Variant.objects.values_list('snpeff_effect').distinct().order_by('snpeff_effect')

    effect = forms.MultipleChoiceField(choices=CHOICES, required=False)
    
    CHOICES = ['NONE', 'SILENT', 'MISSENSE', 'NONSENSE']
     
    #[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_functional_class').distinct().exclude(snpeff_functional_class='').order_by('snpeff_functional_class')]
    func_class = forms.MultipleChoiceField(choices=CHOICES, required=False)
    

    CHOICES = ['HIGH', 'MODERATE', 'MODIFIER', 'LOW']

    #[(x[0], x[0]) for x in Variant.objects.values_list('snpeff_impact').distinct().order_by('snpeff_impact')]

    impact = forms.MultipleChoiceField(choices=CHOICES, required=False)
    
    CHOICES = (('<', '<= '),
               ('>', '>= '),
               ('=', '='),)
    
    dbsnp_option = forms.ChoiceField(required=False, choices=CHOICES, label="dbSNP Build")
    dbsnp_build = forms.CharField(max_length=50, required=False, label="DbsNP Build")
    
    read_depth_option = forms.ChoiceField(required=False, choices=CHOICES, label="Read Depth", initial='>')
    read_depth = forms.CharField(max_length=50, required=False, label="", initial='10')
    
    variants_per_gene_option = forms.ChoiceField(required=False, choices=CHOICES, label="Variants per Gene")
    variants_per_gene = forms.CharField(max_length=50, required=False, label="")
    
    #Extra options
    genes_in_common = forms.BooleanField(required=False, label="Show only variants present in genes from all the individuals selected", initial=True)
    dbsnp = forms.BooleanField(required=False, label="Show only variants with rsID in dbSNP")
    cln = forms.BooleanField(required=False, label="Show only variants with clinical association")
    exclude_segdup = forms.BooleanField(required=False, label="Exclude variants in regions of Segmental Duplications", initial=True)
    

class FilterWiZardForm3(forms.Form):
    #100genomes freqeuncy
    CHOICES = (('<', '<= '),
               ('>', '>= '),
               ('=', '='),)
    #1000Genomes
    # genomes1000_option = forms.ChoiceField(required=False, choices=CHOICES, label="1000genomes Frequency")
    genomes1000 = forms.CharField(max_length=50, required=False, label="1000genomes Frequency", initial="0 - 0.005")
    genomes1000_exclude = forms.BooleanField(required=False, label="Exclude all variants present in 1000genomes")
    #DBSNP Frequency
    # dbsnp_freq_option = forms.ChoiceField(required=False, choices=CHOICES, label="DBSNP Frequency")
    dbsnp_frequency = forms.CharField(max_length=50, required=False, label="", initial="0 - 0.005")
    dbsnp_exclude = forms.BooleanField(required=False, label="Exclude all variants present in dbSNP")
    
    #Variation Server Frequency
    # variationserver_option = forms.ChoiceField(required=False, choices=CHOICES, label="ESP5400 Frequency")
    esp_frequency = forms.CharField(max_length=50, required=False, label="", initial="0 - 0.005")
    esp_exclude = forms.BooleanField(required=False, label="Exclude all variants present in Exome Sequencing Project")
    
    # sift_option = forms.ChoiceField(required=False, choices=CHOICES, label="Sift Option")
    sift = forms.CharField(max_length=50, required=False, label="Sift Frequency", initial="0 - 1.0")
    sift_exclude = forms.BooleanField(required=False, label="Exclude variants without sift score", initial=False)
    
    #POLYPHEN
    # polyphen_option = forms.ChoiceField(required=False, choices=CHOICES, label="Polyphen2 Option", initial='>')
    polyphen = forms.CharField(max_length=50, required=False, label="Polyphen2 Frequency", initial='0 - 1.0')
    polyphen_exclude = forms.BooleanField(required=False, label="Exclude variants without polyphen score", initial=False)
    
    mendelmd_score = forms.CharField(max_length=50, required=False, label="Mendel,MD Score", initial='3 - 12')
    
    



from django.core.management.base import BaseCommand, CommandError
# from polls.models import Poll
from genes.models import *
import codecs

from diseases.models import *

from genes.models import Gene as Gene2

import os
import zipfile

def populate_genes():
    #populate genes from hgnc
    Gene2.objects.all().delete()

    # zip_file    = zipfile.ZipFile(sys.argv[1])
    zip = zipfile.ZipFile('data/hgnc/hgnc_complete_set.txt.zip', 'r')
    hgnc = zip.open('hgnc_complete_set.txt', 'r')
    # hgnc = codecs.open('data/hgnc/hgnc_complete_set.txt', 'r', "ISO-8859-1")
    header = hgnc.readline().decode("utf-8").strip().split('\t')
    count_gene = 1
    
    genes = []
    countline = 0
    for lineb in hgnc:
#        print count_gene
        line = lineb.decode("utf-8")

        count_gene += 1
        countline += 1
        #print countline
        if count_gene == 5000:
            print(countline)
            #bulk_insert(genes, show_sql=False)
            Gene2.objects.bulk_create(genes)
            count_gene = 0
            genes = []
        
#        print line
        line = line.strip().split('\t')
        # print(len(line))
        # print(line)
        #ugly hack, to fill missing fields ?
        if len(line) != 48:
            count = 48 - len(line) 
            for item in range(0,count):
                line.append('')
        
        gene = Gene2(
                    hgnc_id = line[0],
                    symbol = line[1],
                    name = line[2],
                    locus_group = line[3],
                    locus_type = line[4],
                    status = line[5],
                    location = line[6],
                    location_sortable = line[7],
                    alias_symbol = line[8],
                    alias_name = line[9],
                    prev_symbol = line[10],
                    prev_name = line[11],
                    gene_family = line[12],
                    gene_family_id = line[13],
                    date_approved_reserved = line[14],
                    date_symbol_changed = line[15],
                    date_name_changed = line[16],
                    date_modified = line[17],
                    entrez_id = line[18],
                    ensembl_gene_id = line[19],
                    vega_id = line[20],
                    ucsc_id = line[21],
                    ena = line[22],
                    refseq_accession = line[23],
                    ccds_id = line[24],
                    uniprot_ids = line[25],
                    pubmed_id = line[26],
                    mgd_id = line[27],
                    rgd_id = line[28],
                    lsdb = line[29],
                    cosmic = line[30],
                    omim_id = line[31],
                    mirbase = line[32],
                    homeodb = line[33],
                    snornabase = line[34],
                    bioparadigms_slc = line[35],
                    orphanet = line[36],
                    pseudogene_org = line[37],
                    horde_id = line[38],
                    merops = line[39],
                    imgt = line[40],
                    iuphar = line[41],
                    kznf_gene_catalog = line[42],
                    mamit_trnadb = line[43],
                    cd = line[44],
                    lncrnadb = line[45],
                    enzyme_id = line[46],
                    intermediate_filament_db = line[47]
                )
    
        
        genes.append(gene)

    Gene2.objects.bulk_create(genes)
    #bulk_insert(genes, show_sql=False)
    hgnc.close()

def populate_diseases():
    # print os.getcwd()

    Gene.objects.all().delete()
    Disease.objects.all().delete()
   
    if os.path.isfile('data/omim/morbidmap.txt'):

        morbidmap = open('data/omim/morbidmap.txt', 'r')
        for line in morbidmap:
            # print(line)
            if not line.startswith('#'):
                line = line.split('\t')
                name = line[0]
                omim_id = line[2]
                chr_location = line[3].strip()
                gene_names = line[1]
                disease = Disease.objects.create(name=name, omim_id=omim_id, chr_location=chr_location, gene_names=gene_names)
                disease.save()
                genes = line[1].split(',')
                
                
                gene_officialname = genes[0].strip()
                
                chr_candidate = line[3].split('q')[0]
                chr_candidate2 = line[3].split('p')[0]
                if len(chr_candidate) < len(chr_candidate2):
                    chromossome = chr_candidate
                else:
                    chromossome = chr_candidate2
                #add gene relation with disease
                if gene_names != '':
                    #check if gene exists otherwise create gene
                    try:
                        gene = Gene.objects.get(names=gene_names, official_name=gene_officialname, chromossome=chromossome)
                        gene.diseases.add(disease)
                        gene.save()
                    except Gene.DoesNotExist:
                        gene = Gene.objects.create(names=gene_names, official_name=gene_officialname, chromossome=chromossome)
                        gene.save()
                        gene.diseases.add(disease)
                        gene.save()
                        
                 
                    #same for genes_Gene
                    #check if gene exists for each alternative name of gene
                    gene_names = gene_names.strip().split(',')
                    try:
                        gene = Gene2.objects.get(symbol=gene_officialname)
                        gene.diseases.add(disease)
                        gene.save()
                    except Gene2.DoesNotExist:
                        pass
                        
        morbidmap.close()

def populate_CGD():

    CGDCondition.objects.all().delete()
    Manifestation.objects.all().delete()
    Intervention.objects.all().delete()
    CGDEntry.objects.all().delete()


    zip = zipfile.ZipFile('data/CGD/CGD.txt.zip', 'r')
    CDGfile = zip.open('CGD.txt', 'r')

    # CDGfile = open('data/CGD/CGD.txt')
    header = next(CDGfile)
    for lineb in CDGfile:
        line = lineb.decode('utf-8')
        gene_item = line.split('\t')
        # print gene_item
        cgd = CGDEntry()

        cgd.GENE = gene_item[0]
        cgd.HGNC_ID = gene_item[1]
        cgd.ENTREZ_GENE_ID = gene_item[2]
        
        cgd.INHERITANCE = gene_item[4]
        cgd.AGE_GROUP = gene_item[5]
        cgd.ALLELIC_CONDITIONS = gene_item[6]

        cgd.COMMENTS = gene_item[9]
        cgd.INTERVENTION_RATIONALE = gene_item[10]
        
        references_list =[]
        references = gene_item[11].split(';')
        for ref in references:
            references_list.append(ref.strip())

        cgd.REFERENCES = references_list

        cgd.save()

        conditions = gene_item[3].split(';')
        for condition in conditions:
            condition = condition.strip()
            if condition != '':
                cond_obj = CGDCondition.objects.get_or_create(name=condition)[0]
                cgd.CONDITIONS.add(cond_obj)
                # cgd.CONDITIONS.append(cond_obj)

        manifestations = gene_item[7].split(';')
        for manifestation in  manifestations:
            #try get manifestation
            manifestation = manifestation.strip()
            if manifestation != '':
                man_obj = Manifestation.objects.get_or_create(name=manifestation)[0]
                cgd.MANIFESTATION_CATEGORIES.add(man_obj)
                # cgd.MANIFESTATION_CATEGORIES.append(man_obj)  
                

        interventions = gene_item[8].split(';')
        for intervention in  interventions:
            #try get manifestation
            intervention = intervention.strip()
            int_obj = Intervention.objects.get_or_create(name=intervention)[0]
            cgd.INTERVENTION_CATEGORIES.add(int_obj)
            # cgd.INTERVENTION_CATEGORIES.append(int_obj)

        
class Command(BaseCommand):
    # args = '<poll_id poll_id ...>'
    help = 'Populate data from diseases and genes'

    def handle(self, *args, **options):

        print('Populate Genes!')
        populate_genes()
        print('Populate Diseases from OMIM!')
        populate_diseases()
        #populate diseases from OMIM
        print('Populate Diseases from CGD!')
        populate_CGD()
        #populate dieases from CGD
        #populate dieases from CGD


        # for poll_id in args:
        #     try:
        #         poll = Poll.objects.get(pk=int(poll_id))
        #     except Poll.DoesNotExist:
        #         raise CommandError('Poll "%s" does not exist' % poll_id)

        #     poll.opened = False
        #     poll.save()

        #     self.stdout.write('Successfully closed poll "%s"' % poll_id)



from django.db.models import Q
from diseases.models import *
from diseases.models import Gene as GeneDisease
from genes.models import *
from individuals.models import *
from variants.models import *
from django.shortcuts import get_object_or_404
from django.db.models import Count

from django.http import HttpResponse
import csv
import pickle

from databases.models import VariSNP



#HERE THE FILTERS STARTS  
def filter_individuals_variants(request, query, args, exclude):
    #INDIVIDUALS
    individuals = request.GET.getlist('individuals')
    
    groups = request.GET.getlist('groups')
    # print groups
    individuals_list = []
    if len(groups) > 0:
        for group_id in groups:
            group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
            for individual in group_individuals:
                 individuals_list.append(str(str(individual)))
            print(individuals_list)
    individuals_list = individuals_list + individuals
    print('individuals_list', individuals_list)


    #EXCLUDE individuals
    exclude_individuals = request.GET.getlist('exclude_individuals')
#            print exclude_individuals
    
    #exclude groups append to individuals
    exclude_groups = request.GET.getlist('exclude_groups')
#            print exclude_groups
    exclude_individuals_list = []
    if len(exclude_groups) > 0:
        for group_id in exclude_groups:
            group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
            for individual in group_individuals:
                if str(str(individual)) not in individuals_list:
                    exclude_individuals_list.append(str(str(individual)))
    
    print('exclude_individuals_list', exclude_individuals_list)
            
    exclude_individuals_list = exclude_individuals_list + exclude_individuals
#            print exclude_individuals_list
    exclude_individuals_variants = {}
    #exclude variants from individuals
    if len(exclude_individuals_list) > 0:
        exclude_indexes = Variant.objects.filter(individual__id__in=exclude_individuals_list, *args, **query).exclude(**exclude).values_list('index', flat=True)
        exclude['index__in'] = exclude_indexes
    # print 'return individual list, exxclue', exclude['index__in']
    return individuals_list

def filter_variants_per_gene(request, query, args, exclude):
    # OPTION variants per gene
    variants_per_gene = request.GET.get('variants_per_gene', '')
    
    print('variants per gene')
    print(variants_per_gene)
    if variants_per_gene != '':
        
        variants_per_gene = int(variants_per_gene)
        variants_per_gene_option = request.GET.get('variants_per_gene_option', '')
        
        print('Debugging')
        print(variants_per_gene)
        print(variants_per_gene_option)

        genes_exclude_list = []
        genes_only_list = []
        #this works for 2 indivuduals in list and will show only genes with variants in both option on
        for individual in query['individual_id__in']:
#                        print individual
#                        print query
#                        print exclude
            print('get list of all genes for each individual')
            individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values('gene').exclude(gene="").annotate(count=Count('gene')).distinct() #.aggregate(Count('gene', distinct=True))
            print(len(individual_genes))
            if variants_per_gene_option == '>':    
                for gene in individual_genes:
                    if gene['count'] >= variants_per_gene:
                        genes_only_list.append(gene['gene'])
                    else:
                        # if genes_in_common == 'on':
                        genes_exclude_list.append(gene['gene'])
                            
            elif variants_per_gene_option == '<':                    
                for gene in individual_genes:
                    if gene['count'] <= variants_per_gene:
                        genes_only_list.append(gene['gene'])
                    else:
                        # if genes_in_common == 'on':
                        genes_exclude_list.append(gene['gene'])
            elif variants_per_gene_option == '=':
                for gene in individual_genes:
                    if gene['count'] == variants_per_gene:
                        genes_only_list.append(gene['gene'])
                    else:
                        # if genes_in_common == 'on':
                        genes_exclude_list.append(gene['gene'])
        #remove variants without gene name                    
        args.append(Q(gene__in=genes_only_list))
        args.append(~Q(gene__in=genes_exclude_list))
    
def filter_genes_in_common(request, query, args, exclude):
    #option GENES in COMMON
    print('genes in common')

    genes_in_common = request.GET.get('genes_in_common', '')
    if genes_in_common == 'on':
        #get all genes from individual
        individual_gene_list = []
        for individual in query['individual_id__in']:
            individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('gene', flat=True).exclude(gene="").distinct()
            # print 'genes finished query', len()
            individual_genes = set(list(individual_genes))
            individual_gene_list.append(individual_genes)
        genes_in_common_list = set.intersection(*individual_gene_list)                    
        query['gene__in'] = genes_in_common_list#genes_in_common_list

def filter_positions_in_common(request, query, args, exclude):
    print('positions in common')
    positions_in_common = request.GET.get('positions_in_common', '')
    if positions_in_common == 'on':
        #get all genes from individual
        individual_positions_list = []
        for individual in query['individual_id__in']:
            #should be done with an index ex. 1-2835763-C-T 'DONE :)'
            individual_positions = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('pos', flat=True).distinct()
            individual_positions = set(list(individual_positions))
            individual_positions_list.append(individual_positions)
        positions_in_common_list = set.intersection(*individual_positions_list)                    
        query['pos__in'] = positions_in_common_list#genes_in_common_list

def filter_chr(request, query):
    chr = request.GET.get('chr', '')
    if chr != '':
        query['chr'] = chr

def filter_pos(request, query):
    pos = request.GET.get('pos', '')
    if pos != '':
        pos = pos.split('-')
        if len(pos) == 2:
            query['pos__range'] = (pos[0], pos[1])
        else:
            query['pos'] = pos[0]
       
def filter_snp_list(request, query, exclude):
    snp_list = request.GET.get('snp_list', '')
    snp_list = snp_list.split('\r\n')

    if snp_list[0] != '':
        safe_snp_list = []
        for row in snp_list:
            row = row.split(',')
            for item in row:
                safe_snp_list.append(item.strip())
        query['variant_id__in'] = safe_snp_list
    #exclude snp_list
    exclude_snp_list = request.GET.get('exclude_snp_list', '')
    exclude_snp_list = exclude_snp_list.split('\r\n')
    if exclude_snp_list[0] != '':
        safe_exclude_snp_list = []
        for row in exclude_snp_list:
            row = row.split(',')
            for item in row:
                safe_exclude_snp_list.append(item.strip())
        exclude['variant_id__in']=safe_exclude_snp_list#args.append(~Q(variant_id__in=safe_snp_list))

def filter_gene_list(request, query, args):
    #Gene List
    gene_list = request.GET.get('gene_list', '')
    gene_list = gene_list.split('\r\n')
#    print 'gene_list'
    if gene_list[0] != '':
        safe_gene_list = []
        for row in gene_list:
            row = row.split(',')
            for item in row:
                safe_gene_list.append(item.strip().upper())
        print(safe_gene_list)
        query['gene__in'] = safe_gene_list
    
    exclude_gene_list = request.GET.get('exclude_gene_list', '')
    exclude_gene_list = exclude_gene_list.split('\r\n')
    if exclude_gene_list[0] != '':
        safe_gene_list = []
        for row in exclude_gene_list:
            row = row.split(',')
            for item in row:
                safe_gene_list.append(item.strip().upper())
        print(safe_gene_list)
        args.append(~Q(gene__in=safe_gene_list))
    
def filter_mutation_type(request, args):

    genotype = request.GET.get('genotype', '')

    if genotype != '':
        print('genotype', genotype)
        args.append(Q(genotype=genotype))


    mutation_type = request.GET.get('mutation_type', '')
    if mutation_type == 'homozygous':
        # genotypes = ['0/0', './.', '0/1', '1/0', '0/2', '2/0']
        args.append(Q(mutation_type='HOM'))
    elif mutation_type == 'heterozygous':
        #genotypes = ['0/0', './.', '1/1', '2/1', '1/2', '2/2']
        args.append(Q(mutation_type='HET'))

def filter_effect(request, query):
    effect = request.GET.getlist('effect')
    if len(effect) > 0:
        # query['snp_eff__in'] = variant_type
        print('effect', effect)
        query['snpeff_effect__in'] = effect


def filter_dbsnp(request, query):
    dbsnp = request.GET.get('dbsnp', '')
    if dbsnp == 'on':
        query['variant_id'] = "."

def filter_varisnp(request, query, exclude):

    exclude_varisnp = request.GET.get('exclude_varisnp', '')
    if exclude_varisnp == 'on':

        snp_list = VariSNP.objects.all().values_list('dbsnp_id', flat=True)
        
        # safe_snp_list = []
        # for snp in snp_list:
        #     safe_snp_list.append(str(snp))
        # print 'snp_list', len(snp_list), snp_list[:5]
        # snp_list = list(snp_list)
        # print 'snp_list depois', len(snp_list), snp_list[:5]


        if 'variant_id__in' in exclude:
            # print 'varisnp variant_id__in ja existe!'
            # print snp_list
            exclude['variant_id__in'].extend(safe_snp_list)
        else:
            exclude['variant_id__in'] = safe_snp_list

def filter_by_1000g(request, args):
    genomes1000_exclude = request.GET.get('genomes1000_exclude', '')
    if genomes1000_exclude == 'on':
        args.append(Q(genomes1k_maf__isnull=True))
    else:
        genomes1000 = request.GET.get('genomes1000', '')
        if genomes1000 != '':
            genomes1000 = genomes1000.split(' - ')
            if len(genomes1000) == 2:
                min = float(genomes1000[0]) 
                max = float(genomes1000[1])
                args.append((Q(genomes1k_maf__lte=max) & Q(genomes1k_maf__gte=min)) | Q(genomes1k_maf__isnull=True))
            if len(genomes1000) == 1:
                max = float(genomes1000[0])
                args.append((Q(genomes1k_maf__lte=max) & Q(genomes1k_maf__gte=0)) | Q(genomes1k_maf__isnull=True))

def filter_by_dbsnp(request, args):
    dbsnp_exclude = request.GET.get('dbsnp_exclude', '')
    if dbsnp_exclude == 'on':
        args.append(Q(dbsnp_maf__isnull=True))
    else:
        dbsnp = request.GET.get('dbsnp_frequency', '')
        if dbsnp != '':
            dbsnp = dbsnp.split(' - ')
            if len(dbsnp) == 2:
                min = float(dbsnp[0]) 
                max = float(dbsnp[1])
                args.append((Q(dbsnp_maf__lte=max) & Q(dbsnp_maf__gte=min)) | Q(dbsnp_maf__isnull=True))    
            if len(dbsnp) == 1:
                max = float(dbsnp[0])
                args.append((Q(dbsnp_maf__lte=max) & Q(dbsnp_maf__gte=0)) | Q(dbsnp_maf__isnull=True))    

def filter_by_esp(request, args):
    esp_exclude = request.GET.get('esp_exclude', '')
    if esp_exclude == 'on':
        args.append(Q(esp_maf__isnull=True))
    else:
        esp = request.GET.get('esp_frequency', '')
        if esp != '':
            esp = esp.split(' - ')
            min = float(esp[0]) 
            max = float(esp[1])
            args.append((Q(esp_maf__lte=max) & Q(esp_maf__gte=min)) | Q(esp_maf__isnull=True))     
               
# def filter_genomes1000(request, args):
#     #1000genomes
#     genomes1000 = request.GET.get('genomes1000', '')
#     if genomes1000 != '':
#         genomes1000_option = request.GET.get('genomes1000_option', '')
#         if genomes1000_option == '<':
#             args.append(Q(genomes1k_maf__lte=float(genomes1000)) | Q(genomes1k_maf__isnull=True))
#         elif genomes1000_option == '>':
#             args.append(Q(genomes1k_maf__gte=float(genomes1000)) | Q(genomes1k_maf__isnull=True))
#         elif genomes1000_option == '=':
#             args.append(Q(genomes1k_maf=float(genomes1000)))

# def filter_dbsnp_frequency(request, args):
#     #dbsnp Freq
#     dbsnp_frequency = request.GET.get('dbsnp_frequency', '')
#     if dbsnp_frequency != '':
#         dbsnp_freq_option = request.GET.get('dbsnp_freq_option', '')
#         if dbsnp_freq_option == '<':
#             args.append(Q(dbsnp_maf__lte=float(dbsnp_frequency)) | Q(dbsnp_maf__isnull=True))
#         elif dbsnp_freq_option == '>':
#             args.append(Q(dbsnp_maf__gte=float(dbsnp_frequency)) | Q(dbsnp_maf__isnull=True))
#         elif dbsnp_freq_option == '=':    
#             args.append(Q(dbsnp_maf=float(dbsnp_frequency)))
# def filter_variationserver_frequency(request, args):
#     #Exome Variation Server Freq
#     variationserver_frequency = request.GET.get('variationserver_frequency', '')
#     if variationserver_frequency != '':
#         variationserver_option = request.GET.get('variationserver_option', '')
#         if variationserver_option == '<':
#             args.append(Q(esp_maf__lte=float(variationserver_frequency)) | Q(esp_maf__isnull=True))
#         elif variationserver_option == '>':
#             args.append(Q(esp_maf__gte=float(variationserver_frequency)) | Q(esp_maf__isnull=True))
#         elif variationserver_option == '=':
#             args.append(Q(esp_maf=float(variationserver_frequency)))


def filter_by_individuals(request, args, query, exclude):
    #INDIVIDUALS
    individuals = request.GET.getlist('individuals')
    
    groups = request.GET.getlist('groups')
    # print groups
    individuals_list = []
    if len(groups) > 0:
        for group_id in groups:
            group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
            for individual in group_individuals:
                 individuals_list.append(str(str(individual)))
            print(individuals_list)
    individuals_list = individuals_list + individuals
    print('individuals_list', individuals_list)


    #EXCLUDE individuals
    exclude_individuals = request.GET.getlist('exclude_individuals')
#            print exclude_individuals
    
    #exclude groups append to individuals
    exclude_groups = request.GET.getlist('exclude_groups')
#            print exclude_groups
    exclude_individuals_list = []
    if len(exclude_groups) > 0:
        for group_id in exclude_groups:
            group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
            for individual in group_individuals:
                if str(str(individual)) not in individuals_list:
                    exclude_individuals_list.append(str(str(individual)))
    
    print('exclude_individuals_list', exclude_individuals_list)
            
    exclude_individuals_list = exclude_individuals_list + exclude_individuals
#            print exclude_individuals_list
    exclude_individuals_variants = {}
    #exclude variants from individuals
    if len(exclude_individuals_list) > 0:
        exclude_indexes = Variant.objects.filter(individual__id__in=exclude_individuals_list, *args, **query).exclude(**exclude).values_list('index', flat=True)
        exclude['index__in'] = exclude_indexes


    if len(individuals_list) > 0:
        #only add to query after filtering the indexes 
        query['individual_id__in'] = individuals_list
        # OPTION variants per gene
        variants_per_gene = request.GET.get('variants_per_gene')
        #option GENES in COMMON
        genes_in_common = request.GET.get('genes_in_common', '')

        print('variants per gene')
        print(variants_per_gene)
        if variants_per_gene != '':
            
            variants_per_gene = int(variants_per_gene)
#                print 'Variants per gene'
            variants_per_gene_option = request.GET.get('variants_per_gene_option', '')
            
            print('Debugging')
            print(variants_per_gene)
            print(variants_per_gene_option)

            genes_exclude_list = []
            genes_only_list = []
            #this works for 2 indivuduals in list and with show only genes with variants in both option on
            for individual in individuals_list:
#                        print individual
#                        print query
#                        print exclude
                print('get list of all genes for each individual')
                individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values('gene').exclude(gene="").annotate(count=Count('gene')).distinct() #.aggregate(Count('gene', distinct=True))
                print(len(individual_genes))
                if variants_per_gene_option == '>':    
                    for gene in individual_genes:
                        if gene['count'] >= variants_per_gene:
                            genes_only_list.append(gene['gene'])
                        else:
                            # if genes_in_common == 'on':
                            genes_exclude_list.append(gene['gene'])
                                
                elif variants_per_gene_option == '<':                    
                    for gene in individual_genes:
                        if gene['count'] <= variants_per_gene:
                            genes_only_list.append(gene['gene'])
                        else:
                            # if genes_in_common == 'on':
                            genes_exclude_list.append(gene['gene'])
                elif variants_per_gene_option == '=':
                    for gene in individual_genes:
                        if gene['count'] == variants_per_gene:
                            genes_only_list.append(gene['gene'])
                        else:
                            # if genes_in_common == 'on':
                            genes_exclude_list.append(gene['gene'])
            #remove variants without gene name                    
            args.append(Q(gene__in=genes_only_list))
            args.append(~Q(gene__in=genes_exclude_list))
        
        if genes_in_common == 'on':
            #get all genes from individual
            individual_gene_list = []
            for individual in individuals_list:
                individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('gene', flat=True).exclude(gene="").distinct()
                individual_genes = set(list(individual_genes))
                individual_gene_list.append(individual_genes)
            genes_in_common_list = set.intersection(*individual_gene_list)                    
            query['gene__in'] = genes_in_common_list#genes_in_common_list

        
        positions_in_common = request.GET.get('positions_in_common', '')
        if positions_in_common == 'on':
            #get all genes from individual
            individual_positions_list = []
            for individual in individuals_list:
                #should be done with an index ex. 1-2835763-C-T 'DONE :)'
                individual_positions = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('pos', flat=True).distinct()
                individual_positions = set(list(individual_positions))
                individual_positions_list.append(individual_positions)
            positions_in_common_list = set.intersection(*individual_positions_list)                    
            query['pos__in'] = positions_in_common_list#genes_in_common_list

    


    
    


def filter_qual(request, args):
    qual = request.GET.get('qual', '')
    if qual != '':
        qual_option = request.GET.get('qual_option', '')
        if qual_option == '<':
            args.append(Q(qual__lte=float(qual)))
        elif qual_option == '>':
            args.append(Q(qual__gte=float(qual)))
        elif qual_option == '=':
            args.append(Q(qual=float(qual)))


def filter_filter(request, query):     
    filter = request.GET.getlist('filter')
    if len(filter) > 0:
        query['filter__in'] = filter

def filter_by_sift(request, args):
    sift = request.GET.get('sift', '')
    if sift != '':
        sift_exclude = request.GET.get('sift_exclude', '')
        if sift_exclude == 'on':
            sift_flag = True
        else:
            sift_flag = False
        sift = sift.split(' - ')
        sift_min = float(sift[0]) 
        sift_max = float(sift[1])
        if sift_flag:
            args.append(Q(sift__lte=sift_max) & Q(sift__gte=sift_min))
        else:
            args.append((Q(sift__lte=sift_max) & Q(sift__gte=sift_min)) | Q(sift__isnull=True))
def filter_by_cadd(request, args):
    cadd = request.GET.get('cadd', '')
    if cadd != '':
        cadd_exclude = request.GET.get('cadd_exclude', '')
        if cadd_exclude == 'on':
            cadd_flag = True
            # print('CADD Flag on')
        else:
            cadd_flag = False
        cadd = cadd.split(' - ')
        cadd_min = float(cadd[0]) 
        cadd_max = float(cadd[1])
        # print('CADD', cadd_min, cadd_max)
        if cadd_flag:
            args.append(Q(cadd__lte=cadd_max) & Q(cadd__gte=cadd_min))
        else:
            args.append((Q(cadd__lte=cadd_max) & Q(cadd__gte=cadd_min)) | Q(cadd__isnull=True))

def filter_by_mcap(request, args):
    mcap = request.GET.get('mcap', '')
    if mcap != '':
        mcap_exclude = request.GET.get('mcap_exclude', '')
        if mcap_exclude == 'on':
            mcap_flag = True
            # print('CADD Flag on')
        else:
            mcap_flag = False
        mcap = mcap.split(' - ')
        mcap_min = float(mcap[0]) 
        mcap_max = float(mcap[1])
        # print('CADD', cadd_min, cadd_max)
        if mcap_flag:
            args.append(Q(mcap_score__lte=mcap_max) & Q(mcap_score__gte=mcap_min))
        else:
            args.append((Q(mcap_score__lte=mcap_max) & Q(mcap_score__gte=mcap_min)) | Q(mcap_score__isnull=True))
                   
def filter_by_rf_score(request, args):
    rf_score = request.GET.get('rf_score', '')
    if rf_score != '':
        rf_exclude = request.GET.get('rf_exclude', '')
        if rf_exclude == 'on':
            rf_flag = True
        else:
            rf_flag = False
        rf = rf_score.split(' - ')
        rf_min = float(rf[0]) 
        rf_max = float(rf[1])
        if rf_flag:
            args.append(Q(rf_score__lte=rf_max) & Q(rf_score__gte=rf_min))
        else:
            args.append((Q(rf_score__lte=rf_max) & Q(rf_score__gte=rf_min)) | Q(rf_score__isnull=True))            
def filter_by_ada_score(request, args):
    ada_score = request.GET.get('ada_score', '')
    if ada_score != '':
        ada_exclude = request.GET.get('ada_exclude', '')
        if ada_exclude == 'on':
            ada_flag = True
        else:
            ada_flag = False
        ada = ada_score.split(' - ')
        ada_min = float(ada[0]) 
        ada_max = float(ada[1])
        if ada_flag:
            args.append(Q(ada_score__lte=ada_max) & Q(ada_score__gte=ada_min))
        else:
            args.append((Q(ada_score__lte=ada_max) & Q(ada_score__gte=ada_min)) | Q(ada_score__isnull=True))            
            
def filter_by_pp2(request, args):
    polyphen = request.GET.get('polyphen', '')
    if polyphen != '':
        polyphen_exclude = request.GET.get('polyphen_exclude', '')
        if polyphen_exclude == 'on':
            polyphen_flag = True
        else:
            polyphen_flag = False
        polyphen = polyphen.split(' - ')
        polyphen_min = float(polyphen[0]) 
        polyphen_max = float(polyphen[1])
        if polyphen_flag:
            args.append(Q(polyphen2__lte=polyphen_max) & Q(polyphen2__gte=polyphen_min))
        else:
            args.append((Q(polyphen2__lte=polyphen_max) & Q(polyphen2__gte=polyphen_min)) | Q(polyphen2__isnull=True))


# def filter_by_hi_score(request, args):
#     hi_exclude = request.GET.get('hi_exclude', '')
#     if hi_exclude == 'on':
#         args.append(Q(hi_score_perc__isnull=False))
    
#     hi = request.GET.get('hi_frequency', '')
#     if hi != '':
#         hi = hi.split(' - ')
#         min = float(hi[0]) 
#         max = float(hi[1])
#         args.append((Q(hi_score_perc__lte=max) & Q(hi_score_perc__gte=min)))
#         #| Q(hi_score_perc__isnull=True)




def filter_by_segdup(request, args):
    exclude_segdup = request.GET.get('exclude_segdup', '')
    if exclude_segdup == 'on':
        args.append(Q(segdup=''))


def filter_cgd(request, args):
    cgdmanifestation = request.GET.getlist('cgdmanifestation')
    # interventions = request.GET.getlist('interventions')
    conditions = request.GET.getlist('cgd')
    # print 'FILTER BY CGD'
    # print manifestations, interventions
    if len(cgdmanifestation) > 0:
        if cgdmanifestation[0] != '':
            #get a list of CGDENTRY
            cgdentries = CGDEntry.objects.filter(MANIFESTATION_CATEGORIES__in=cgdmanifestation)
            gene_list = []
            for gene in cgdentries:
                gene_list.append(gene.GENE)
            args.append(Q(gene__in=gene_list))
    if len(conditions) > 0:
        if conditions[0] != '':
            cgdentries = CGDEntry.objects.filter(CONDITIONS__in=conditions)
            gene_list = []
            for gene in cgdentries:
                gene_list.append(gene.GENE)
            args.append(Q(gene__in=gene_list))



def filter_omim(request, args):
    omim = request.GET.getlist('omim')
    print('omim', omim)
    # print 'FILTER BY CGD'
    # print manifestations, interventions
    print('FILTER OMIM')
    if len(omim) > 0:
        if omim[0] != '':
            omimentries = Disease.objects.filter(id__in=omim)
            gene_list = []
            print('omimentries', omimentries)

            genes = GeneDisease.objects.filter(diseases__in=omimentries)
            print('omimgenes', genes)
            for gene in genes:
                gene_list.append(gene.official_name)
            args.append(Q(gene__in=gene_list))

def filter_hgmd(request, args):
    hgmd = request.GET.getlist('hgmd')
    # print 'omim', omim
    # print 'FILTER BY CGD'
    # print manifestations, interventions
    # print 'FILTER OMIM'
    if len(hgmd) > 0:
        if hgmd[0] != '':
            hgmdentries = HGMDPhenotype.objects.filter(id__in=hgmd)
            gene_list = []
            # print 'hgmdentries', hgmdentries
            genes = HGMDGene.objects.filter(diseases__in=hgmdentries)
            # print 'hgmdgenes', genes
            for gene in genes:
                gene_list.append(gene.symbol)
            args.append(Q(gene__in=gene_list))

def filter_genelists(request, query, args, exclude):
    genelists = request.GET.getlist('genelists')

    safe_gene_list = []
    if len(genelists) > 0:
        
        for genelist_id in genelists:
            genelist_obj = GeneList.objects.get(pk=genelist_id)
            # print 'genelist'
            # print genelist_obj.genes
            gene_list = genelist_obj.genes.split(',')
            for gene in gene_list:
                if gene not in safe_gene_list:
                    safe_gene_list.append(gene.upper())
        query['gene__in'] = safe_gene_list


        # gene_list = gene_list.split('\r\n')
        # # print 'gene_list'
        # if gene_list[0] != u'':
            
        #     for row in gene_list:
        #         row = row.split(',')
        #         for item in row:
        #             gene = item.strip()
        #             if (gene != ' ' and gene != ''):
        #                 if gene not in safe_gene_list:
        #                     safe_gene_list.append(item.strip())
    

    exclude_genelists = request.GET.getlist('exclude_genelists')
    if len(exclude_genelists) > 0:
        exclude_safe_gene_list = []
        for genelist_id in exclude_genelists:
            genelist_obj = GeneList.objects.get(pk=genelist_id)
            gene_list = genelist_obj.genes.split(',')
            for gene in gene_list:
                if gene not in safe_gene_list:
                    if gene not in exclude_safe_gene_list:
                        exclude_safe_gene_list.append(gene)
        if len(exclude_safe_gene_list) > 0:
            exclude['gene__in'] = exclude_safe_gene_list

        # # print 'genelist'
        # # print genelist_obj.genes
        # gene_list = genelist_obj.genes

        # gene_list = gene_list.split('\r\n')
        # # print 'gene_list'
        # if gene_list[0] != u'':
            
        #     for row in gene_list:
        #         row = row.split(',')
        #         for item in row:
        #             gene = item.strip()
        #             if (gene != ' ' and gene != ''):
        #                 if gene not in exclude_safe_gene_list:
        #                     exclude_safe_gene_list.append(item.strip())
    # query['gene__nin'] = safe_gene_list


    # print 'safe_gene_list'
    # print len(safe_gene_list)
    # print safe_gene_list
    # query['gene__in'] = safe_gene_list

    # args.append(Q(gene__in=safe_gene_list))



#HERE STARTS FILTER FOR FAMILY ANALYSIS
def filter_inheritance_option(request):
    inheritance_option = request.GET.get('inheritance_option', '')

    if inheritance_option == '3':
        request.GET.__setitem__('variants_per_gene','2')
        request.GET.__setitem__('variants_per_gene_option','>')
        print(request.GET)
        

def filter_inheritance_option_exclude_individuals(request):
    inheritance_option = request.GET.get('inheritance_option', '')
    exclude_individuals = request.GET.getlist('exclude_individuals')
    father = request.GET.get('father', '')
    mother = request.GET.get('mother', '')
    parents = [father, mother]
    if inheritance_option == '1' or inheritance_option == '2':
        if father not in exclude_individuals:
            request.GET.appendlist('exclude_individuals', father)
        if mother not in exclude_individuals:
            request.GET.appendlist('exclude_individuals', mother)
            
def filter_inheritance_option_mutation_type(request, args):
    inheritance_option = request.GET.get('inheritance_option', '')
    #by exclusion of genotypes
    if inheritance_option == '1':
        genotypes = ['0/0', './.', '0/1', '1/0', '0/2', '2/0']
        args.append(~Q(genotype__in=genotypes))

def filter_sift(request, args):
    sift = request.GET.get('sift', '')
    if sift != '':
        sift_exclude = request.GET.get('sift_exclude', '')
        if sift_exclude == 'on':
            sift_flag = True
        else:
            sift_flag = False
        sift_option = request.GET.get('sift_option', '')
        if sift_option == '<':
            if sift_flag:
                args.append(Q(sift__lte=float(sift)))
            else:
                args.append(Q(sift__lte=float(sift)) | Q(sift__isnull=True))
        elif sift_option == '>':
            if sift_flag:
                args.append(Q(sift__gte=float(sift)))
            else:
                args.append(Q(sift__gte=float(sift)) | Q(sift__isnull=True))
        elif sift_option == '=':
            if sift_flag:
                args.append(Q(sift=float(sift)))
            else:
                args.append(Q(sift=float(sift)) | Q(sift__isnull=True))

def filter_polyphen2(request, args):
    polyphen = request.GET.get('polyphen', '')
    if polyphen != '':
        polyphen_exclude = request.GET.get('polyphen_exclude', '')
        if polyphen_exclude == 'on':
            polyphen_flag = True
        else:
            polyphen_flag = False
            
        polyphen_option = request.GET.get('polyphen_option', '')
        if polyphen_option == '<':
            if polyphen_flag:
                args.append(Q(polyphen2__lte=float(polyphen)))
            else:
                args.append(Q(polyphen2__lte=float(polyphen)) | Q(polyphen2__isnull=True))
        elif polyphen_option == '>':
            if polyphen_flag:
                args.append(Q(polyphen2__gte=float(polyphen)))
            else:
                args.append(Q(polyphen2__gte=float(polyphen)) | Q(polyphen2__isnull=True))
        elif polyphen_option == '=':
            if polyphen_flag:
                args.append(Q(polyphen2=float(polyphen)))
            else:
                args.append(Q(polyphen2=float(polyphen)) | Q(polyphen2__isnull=True))
def filter_dbsnp_build(request, args):
    dbsnp_build = request.GET.get('dbsnp_build', '')
    if dbsnp_build != '':
        dbsnp_option = request.GET.get('dbsnp_option', '')
        if dbsnp_option == '<':
            # build_list = range(1, int(dbsnp_build))
            # T2 = [str(x) for x in build_list]
            # T2.append('')
            # query['dbsnp_build__in'] = #T2
            args.append(Q(dbsnp_build__lte=int(dbsnp_build)) | Q(dbsnp_build__isnull=True))
        elif dbsnp_option == '>':
            # build_list = range(int(dbsnp_build), 138)
            # T2 = [str(x) for x in build_list]
            # T2.append('')
            # query['dbsnp_build__in'] = T2
            args.append(Q(dbsnp_build__gte=int(dbsnp_build)) | Q(dbsnp_build__isnull=True))

def filter_read_depth(request, args):
    read_depth = request.GET.get('read_depth', '')
    if read_depth != '':
        read_depth_option = request.GET.get('read_depth_option', '')
        if read_depth_option == '<':
            args.append(Q(read_depth__lte=int(read_depth)))
        elif read_depth_option == '>':
            args.append(Q(read_depth__gte=int(read_depth)))
        elif read_depth_option == '=':
            args.append(Q(read_depth=int(read_depth)))

                        
def filter_func_class(request, query):
    func_class = request.GET.getlist('func_class')
    if len(func_class) > 0:
        query['snpeff_func_class__in'] = func_class
        # query['effect__in'] = variant_type

def filter_impact(request, query):     
    impact = request.GET.getlist('impact')
    if len(impact) > 0:
        # query['snpeff__impact__in'] = impact
        query['snpeff_impact__in'] = impact
def filter_is_at_hgmd(request, query):     
    # hgmd = request.GET.getlist('is_at_hgmd')
    hgmd = request.GET.get('is_at_hgmd', '')
    if hgmd == 'on':
        query['is_at_hgmd'] = True

def filter_clnsig(request, query):     
    # hgmd = request.GET.getlist('is_at_hgmd')
    clnsig = request.GET.get('clnsig', '')
    if clnsig != '':
        query['clinvar_clnsig'] = clnsig



def export_to_csv(request, variants):
    #export to csv
    export = request.GET.get('export', '')
    if export != '':
        if export == 'csv':
            response = HttpResponse(content_type='text/csv')
            response['Content-Disposition'] = 'attachment; filename=export.csv'
            writer = csv.writer(response)
            
        elif export == 'txt':
            response = HttpResponse(content_type='text/plain')
            response['Content-Disposition'] = 'attachment; filename=export.txt'
            writer = csv.writer(response, delimiter='\t', quoting=csv.QUOTE_NONE)    
        writer.writerow(['Individual', 'Index', 'Pos_index', 'Chr', 'Pos', 'Variant_id', 'Ref', 'Alt', 'Qual', 'Filter', 'Info', 'Format', 'Genotype_col', 'Genotype', 'Read_depth', 'Gene', 'Mutation_type', 'Vartype', 'Genomes1k_maf', 'Dbsnp_maf', 'Esp_maf', 'Dbsnp_build', 'Sift', 'Sift_pred', 'Polyphen2', 'Polyphen2_pred', 'Condel', 'Condel_pred', 'DANN', 'CADD', 'Is_at_omim', 'Is_at_hgmd', 'Hgmd_entries', 'Effect', 'Impact', 'Func_class', 'Codon_change', 'Aa_change', 'Aa_len', 'Gene_name', 'Biotype', 'Gene_coding', 'Transcript_id', 'Exon_rank', 'Genotype_number', 'Allele', 'Gene', 'Feature', 'Feature_type', 'Consequence', 'Cdna_position', 'Cds_position', 'Protein_position', 'Amino_acids', 'Codons', 'Existing_variation', 'Distance', 'Strand', 'Symbol', 'Symbol_source', 'Sift', 'Polyphen', 'Condel']) 
        for variant in variants:
            # print 'variant', variant.index
            writer.writerow([variant.individual, variant.index, variant.pos_index, variant.chr, variant.pos, variant.variant_id, variant.ref, variant.alt, variant.qual, variant.filter, pickle.loads(variant.info), variant.format, variant.genotype_col, variant.genotype, variant.read_depth, variant.gene, variant.mutation_type, variant.vartype, variant.genomes1k_maf, variant.dbsnp_maf, variant.esp_maf, variant.dbsnp_build, variant.sift, variant.sift_pred, variant.polyphen2, variant.polyphen2_pred, variant.condel, variant.condel_pred, variant.dann, variant.cadd, variant.is_at_omim, variant.is_at_hgmd, variant.hgmd_entries])
        return response



from django.shortcuts import render, redirect
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.views.generic.detail import DetailView
from django.contrib.auth.mixins import LoginRequiredMixin

from django.shortcuts import render
from .models import S3Credential, Provider, Profile
from .forms import S3CredentialForm

from django.urls import reverse_lazy

from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

@login_required
def index(request):

    if request.user.is_staff:
        s3_settings = S3Credential.objects.all()
    else:
        s3_settings = S3Credential.objects.filter(user=request.user)

    context = {'s3_settings':s3_settings}
    return render(request, 'settings/index.html', context)

@login_required
def create_s3_credential(request):
    form = S3CredentialForm(request.POST or None)
    if request.method == 'POST':
        if form.is_valid():
            s3credential = S3Credential(user=request.user)
            s3credential.name = form.cleaned_data['name']
            s3credential.access_key = form.cleaned_data['access_key']
            s3credential.secret_key = form.cleaned_data['secret_key']
            s3credential.buckets = form.cleaned_data['buckets']
            s3credential.exclude_paths = form.cleaned_data['exclude_paths']
            s3credential.exclude_files = form.cleaned_data['exclude_files']
            s3credential.save()
            return redirect('settings-index')
    context = {'form': form}
    return render(request, 'settings/create-s3-credential.html', context)

class S3CredentialUpdate(LoginRequiredMixin, UpdateView):
    model = S3Credential
    fields = ['name', 'access_key', 'secret_key', 'buckets', 'exclude_paths', 'exclude_files']

    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects


@method_decorator(login_required, name='dispatch')
class S3CredentialDetailView(DetailView):
    model = S3Credential
    # def get_queryset(self):
    #     return self.model.objects.filter(user=self.request.user)
    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects

@method_decorator(login_required, name='dispatch')
class S3CredentialDelete(DeleteView):
    model = S3Credential
    success_url = reverse_lazy('settings-index')
    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects

class ProviderCreate(CreateView):
    model = Provider
    fields = ['name', ]



from django.conf.urls import *

from django.contrib import admin

from django.views.generic import TemplateView

admin.autodiscover()

from django.conf import settings

from django.conf.urls.static import static

from . import views

import files.views

urlpatterns = [
    # Examples:
    # url(r'^$', 'mendelmd.views.home', name='home'),
    # url(r'^blog/', include('blog.urls')),
    url(r'^$', views.index, name='index'),

    url(r'^docs/$', views.docs, name="docs"),


    url(r'^admin/', admin.site.urls),

    url(r'^upload/', files.views.upload, name='upload'),

    url(r'^accounts/', include('allauth.urls')),
    url(r'^dashboard/', include('dashboard.urls')),
    url(r'^individuals/', include('individuals.urls')),
    url(r'^diseases/', include('diseases.urls')),
    url(r'^genes/', include('genes.urls')),
    url(r'^variants/', include('variants.urls')),
    url(r'^cases/', include('cases.urls')),
    url(r'^filter_analysis/', include('filter_analysis.urls')),
    # url(r'^pathway_analysis/', include('apps.pathway_analysis.urls')),
    url(r'^statistics/', include('stats.urls')),
    url(r'^databases/', include('databases.urls')),
    url(r'^projects/', include('projects.urls')),
    url(r'^select2/', include('django_select2.urls')),
    url(r'^files/', include('files.urls')),
    url(r'^samples/', include('samples.urls')),
    url(r'^settings/', include('settings.urls')),
    url(r'^tasks/', include('tasks.urls')),
    url(r'^workers/', include('workers.urls')),
    url(r'^analyses/', include('analyses.urls')),
    url(r'^apps/', include('mapps.urls')),

] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)

# if settings.DEBUG:
#     import debug_toolbar
#     urlpatterns += patterns('',
#         url(r'^__debug__/', include(debug_toolbar.urls)),
#     )



from django.conf.urls import *

from individuals.views import IndividualDeleteView, GroupDeleteView
from django.contrib.admin.views.decorators import staff_member_required

from . import views

urlpatterns = [
    url(r'^create/$', views.create, name='individual_create'),
    url(r'^edit/(?P<individual_id>[0-9]+)/$', views.edit, name='individual_edit'),
    url(r'^view/(?P<individual_id>\d+)/$', views.view, name='individual_view'),
    url(r'^browse/(?P<individual_id>\d+)/$', views.browse, name='individual_browse'),
    url(r'^delete/(?P<pk>\d+)$', staff_member_required(IndividualDeleteView.as_view()), {}, 'individual_delete'),
    url(r'^annotate/(?P<individual_id>\d+)/$', views.annotate, name='individual_annotate'),
    url(r'^populate/(?P<individual_id>\d+)/$', views.populate, name='individual_populate'),
	url(r'^populate_mongo/(?P<individual_id>\d+)/$', views.populate_mongo, name='individual_populate_mongo'),
    url(r'^$', views.list, name='individuals_list'),
	url(r'^download/(?P<individual_id>\d+)/$', views.download, name='individual_download'),
    url(r'^download_annotated/(?P<individual_id>\d+)/$', views.download_annotated, name='individual_download_annotated'),
    url(r'^create_group/$', views.create_group, name='create_group'),
    url(r'^view_group/(?P<group_id>\d+)/$', views.view_group, name='view_group'),
    url(r'^delete_group/(?P<pk>\d+)$', GroupDeleteView.as_view(), {}, 'group_delete'),
    url(r'^comparison/$', views.comparison, name='comparison'),
    
]


# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('tasks', '__first__'),
        ('projects', '0001_initial'),
        ('files', '0001_initial'),
        ('mapps', '__first__'),
        ('samples', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Analysis',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('params', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                ('name', models.CharField(max_length=30)),
                ('status', models.TextField(blank=True, null=True)),
                ('apps', models.ManyToManyField(to='mapps.App')),
                ('files', models.ManyToManyField(to='files.File')),
                ('project', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='projects.Project')),
                ('samples', models.ManyToManyField(to='samples.Sample')),
                ('tasks', models.ManyToManyField(to='tasks.Task')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'analyses',
            },
        ),
        migrations.CreateModel(
            name='AnalysisType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('repository', models.CharField(blank=True, max_length=600, null=True)),
            ],
            options={
                'verbose_name_plural': 'analysis_types',
            },
        ),
    ]



from django.db import models

# Create your models here.
class Disease(models.Model):
    name = models.CharField(max_length=255)
    omim_id = models.CharField(max_length=255)
    chr_location = models.CharField(max_length=255)
    gene_names = models.CharField(max_length=255)

    def __str__(self):
        return self.name

class Gene(models.Model):
    official_name = models.CharField(max_length=255)
    chromossome = models.CharField(max_length=255, null=True, blank=True)
    names = models.TextField(null=True, blank=True)
    strand = models.CharField(max_length=255, null=True, blank=True)
    chr_location = models.CharField(max_length=255, null=True, blank=True)
    transcription_start = models.IntegerField(null=True, blank=True)
    transcription_end = models.IntegerField(null=True, blank=True)
    cds_start = models.IntegerField(null=True, blank=True)
    cds_end = models.IntegerField(null=True, blank=True)
    exons_count = models.CharField(max_length=500, null=True, blank=True)
    exons_start = models.TextField(null=True, blank=True)
    exons_end = models.TextField(null=True, blank=True)
    diseases = models.ManyToManyField(Disease)


class HGMDPhenotype(models.Model):
    name = models.CharField(max_length=255)
    def __str__(self):
        return self.name
 

class HGMDGene(models.Model):
    symbol = models.CharField(max_length=255)
    aliases = models.TextField(null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    description_aliases = models.TextField(null=True, blank=True)
    location = models.CharField(max_length=255)
    n_mutations = models.IntegerField(null=True, blank=True)
    diseases = models.ManyToManyField(HGMDPhenotype)


#missense/nonsense   splicing    regulatory  small deletions small insertions    small indels    
#gross deletions gross insertions    complex rearrangements  repeat variations



class HGMDMutation(models.Model):
    
    gene = models.ForeignKey(HGMDGene, on_delete=models.CASCADE)

    mutation_type = models.CharField(max_length=255, null=True, blank=True)
    acession = models.CharField(max_length=255, null=True, blank=True)
    
    phenotype = models.ForeignKey(HGMDPhenotype, on_delete=models.CASCADE)
    reference = models.TextField(null=True, blank=True)
    extras = models.TextField(null=True, blank=True)

    rsid = models.CharField(max_length=255, null=True, blank=True)
    dm_mutation = models.BooleanField(default=False)
    coordinate = models.CharField(max_length=100, null=True, blank=True)
    chromossome = models.CharField(max_length=100, null=True, blank=True)

    position = models.CharField(max_length=100, null=True, blank=True)
    #misense
    codon_change = models.CharField(max_length=255, null=True, blank=True)
    aa_change = models.CharField(max_length=255, null=True, blank=True)
    hgvs_nucleotide = models.CharField(max_length=100, null=True, blank=True)
    hgvs_protein = models.CharField(max_length=100, null=True, blank=True)
    #splicing
    splicing_mutation = models.CharField(max_length=255, null=True, blank=True)
    #regulatory
    regulatory_sequence = models.TextField(null=True, blank=True)
    #small deletions
    deletion_sequence = models.TextField(null=True, blank=True)
    #small insertions
    insertion_sequence = models.TextField(null=True, blank=True)

    #gross deletions
    dna_level = models.CharField(max_length=255, null=True, blank=True)
    description = models.CharField(max_length=255, null=True, blank=True)
    #Gross insertions 
    insertion_duplication = models.CharField(max_length=255, null=True, blank=True)
    #Repeat variations
    amplified_sequence = models.CharField(max_length=255, null=True, blank=True)
    location = models.CharField(max_length=255, null=True, blank=True)
    normal_range = models.CharField(max_length=255, null=True, blank=True)
    pathological_range = models.CharField(max_length=255, null=True, blank=True)

    def get_fields(self):
         return [(field, field.value_to_string(self)) for field in HGMDMutation._meta.fields]


    
    



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from celery import shared_task

from files.models import File
from .models import Task as Taskobj

import urllib.request, os

from projects.models import ProjectFile

from django.conf import settings  # noqa

from celery import Celery
app = Celery('mendelmd')

# Using a string here means the worker will not have to
# pickle the object when using Windows.
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)

from tasks.models import Task
from workers.models import Worker
from django.db.models import Q
from workers.tasks import launch_worker, launch_workers, terminate_workers

import os
from subprocess import run, check_output
import subprocess

from individuals.tasks import parse_vcf
from individuals.models import Individual
from variants.models import Variant

from django.core.mail import send_mail

import zipfile
import gzip
import datetime
import time
import socket
import json

from urllib.parse import urlparse

from ftplib import FTP, FTP_TLS
import ftplib

from mapps.models import App
from helpers import b2_wrapper
from helpers.aws_wrapper import AWS

b2 = b2_wrapper.B2()

from multiprocessing import Lock
L = Lock()


@shared_task()
def get_file(file):
    if file.location.startswith('ftp://'):

        basename = os.path.basename(file.location)
        if not os.path.exists('input/{}'.format(basename)):
            command = 'wget -P input/ {}'.format(file.location)
            run(command, shell=True)

            command = 'md5sum input/{}'.format(basename)
            output = check_output(command, shell=True).decode('utf-8').split()[0]

            file.md5 = output
            #upload to b2
            command = 'b2 upload_file mendelmd input/{} files/{}/{}'.format(basename, file.id, basename)
            output = check_output(command, shell=True)
            
            print(output.decode('utf-8'))
            
            file.params = output.decode('utf-8')
            file.url = file.location
            file.location = 'b2://mendelmd/files/{}/{}'.format(file.id, basename)
            file.save()
    elif file.location.startswith('b2://'):

        basename = os.path.basename(file.location)

        if not os.path.exists('input/{}'.format(basename)):

            b2_location = file.location.replace('b2://mendelmd/','')
            command = 'b2 download-file-by-name mendelmd {} input/{}'.format(b2_location, basename)
            output = check_output(command, shell=True)
            print(output.decode('utf-8'))


    return(file)
    # file = File.objects.get(pk=project_file_id)
    # link = file.location

def calculate_md5(path):
    md5_dict = {}
    files = os.listdir(path)
    for file in files:
        command = 'md5sum output/{}'.format(file)
        output = check_output(command, shell=True).decode('utf-8').split()[0]
        file_md5 = output
        md5_dict[file_md5] = file
    return(md5_dict)

@shared_task()
def task_run_task(task_id):
    print('RUN TASK: ', task_id)
    log_output = ''

    task = Task.objects.get(id=task_id)
    
    task.output = ''
    
    start = datetime.datetime.now()
    manifest = task.manifest
    task.machine = socket.gethostbyname(socket.gethostname())
    task.status = 'running'
    task.started = start
    task.save()

    worker = Worker.objects.filter(ip=socket.gethostbyname(socket.gethostname())).reverse()[0]
    worker.n_tasks += 1 
    worker.status = 'running task %s' % (task.id)
    worker.started = start
    worker.save()


    task_location = '/projects/tasks/%s/' % (task.id)
    command = 'mkdir -p %s' % (task_location)
    run(command, shell=True)

    command = 'mkdir -p %s/input' % (task_location)
    run(command, shell=True)

    command = 'mkdir -p %s/output' % (task_location)
    run(command, shell=True)

    command = 'mkdir -p %s/scripts' % (task_location)
    run(command, shell=True)


    os.chdir(task_location)

    with open('manifest.json', 'w') as fp:
        json.dump(manifest, fp, sort_keys=True,indent=4)
    # file_list = []

    # for file_id in manifest['files']:        
    #     print(file_id)
    #     file = File.objects.get(pk=file_id)        
    #     file = get_file(file)
        # file_list.append(file.name)

    #start analysis
    for analysis_name in manifest['analysis_types']:
        print('analysis_name', analysis_name)
        analysis = App.objects.filter(name=analysis_name)[0]
        print(analysis)

        
        command = 'mkdir -p /projects/programs/'
        run(command, shell=True)
        os.chdir('/projects/programs/')

        basename = os.path.basename(analysis.repository)
        print('basename', basename)
        
        command = 'git clone {}'.format(analysis.source)
        run(command, shell=True)

        os.chdir(basename)

        # install
        command = 'bash scripts/install.sh'
        output = check_output(command, shell=True)

        log_output += output.decode('utf-8')
        #run
        
        os.chdir(task_location)
        command = 'python /projects/programs/{}/main.py -i {}'.format(basename, ' '.join(manifest['files']))
        print(command)
        output = run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        log_output += output.stdout.decode('utf-8')


    AWS.upload(task_location+'/output', task.id)

    #upload results to b2/s3
    # md5_dict = calculate_md5('output/')
    # for hash in md5_dict:
    #     print(hash)
    #     try:
    #         file = File.objects.get(md5=hash)
    #     except:
    #         pass
    #         file = File(user=task.user)
    #         file.md5 = hash
    #         file.name = md5_dict[hash]
    #         file.save()

    #         source = 'output/{}'.format(file.name)
    #         dest = 'files/{}/{}'.format(file.id, file.name)

    #         output = b2.upload(source, dest)
            
    #         file.params = output
    #         file.location = 'b2://mendelmd/files/{}/{}'.format(file.id, file.name)
    #         file.save()    
    #         # if task.analysis:
    #         #     task.analysis_set.all()[0].files.add(file)
    #     task.files.add(file)

    # add files if needed :)

    task.status = 'done'
    stop = datetime.datetime.now()
    task.execution_time = str(stop - start)
    task.finished = stop
    task.output = log_output
    task.save()

    worker = Worker.objects.filter(ip=socket.gethostbyname(socket.gethostname())).reverse()[0]
    worker.n_tasks -= 1
    if worker.n_tasks == 0:
        worker.status = 'idle'
    worker.finished = stop
    worker.execution_time = str(stop - start)
    worker.save()
    print('Finished Task %s' % (task.name))

@app.task(queue="qc")
def run_qc(task_id):

    print('RUN QC :D')
    print('task_id', task_id)

    task = Task.objects.get(id=task_id)
    task.status = 'will be running'
    task.save()
    start = datetime.datetime.now()

    manifest = task.manifest

    task.machine = ''
    task.status = 'running'
    task.started = start
    task.save()

    # worker = Worker.objects.filter(ip=socket.gethostbyname(socket.gethostname())).reverse()[0]
    # worker.n_tasks += 1 
    # worker.status = 'running task %s' % (task.id)
    # worker.started = start
    # worker.save()

    task_location = '/projects/tasks/%s/' % (task.id)
    command = 'mkdir -p %s' % (task_location)
    run(command, shell=True)

    os.chdir(task_location)

    with open('manifest.json', 'w') as fp:
        json.dump(manifest, fp, sort_keys=True,indent=4)
    

    task.status = 'done'
    stop = datetime.datetime.now()
    task.execution_time = str(stop - start)
    task.finished = stop
    task.save()

    # worker = Worker.objects.filter(ip=socket.gethostbyname(socket.gethostname())).reverse()[0]
    
    # worker.n_tasks -= 1

    # if worker.n_tasks == 0:
    #     worker.status = 'idle'

    # worker.finished = stop
    # worker.execution_time = str(stop - start)
    # worker.save()

    print('Finished QC %s' % (task.name))


@shared_task()
def import_project_files_task(project_id):
    print('Import Files on ', project_id)

def human_size(bytes, units=[' bytes','KB','MB','GB','TB', 'PB', 'EB']):
    """ Returns a human readable string reprentation of bytes"""
    return str(bytes) + units[0] if bytes < 1024 else human_size(bytes>>10, units[1:])

@shared_task()
def compress_file(task_id):

    task = Taskobj.objects.get(pk=task_id)

    task.status = 'started'
    task.save()

    manifest = task.manifest
    file_id = manifest['file']
    print('File ID', file_id)

    file = File.objects.get(pk=file_id)
        
    if file.location.startswith('/'):

        path = '/'.join(file.location.split('/')[:-1])

        command = 'bgzip {}'.format(file.location)

        output = check_output(command, shell=True)
        
        file.last_output = output.decode('utf-8')
        file.location += '.gz'

    file.save()

    task.status = 'done'
    task.save()

@app.task(queue="annotation")
def annotate_vcf(task_id):

    start = datetime.datetime.now()

    task = Task.objects.get(id=task_id)
    task.machine = socket.gethostbyname(socket.gethostname())
    task.status = 'running'
    task.started = start
    task.save()

    print('Annotate VCF', task.id)
    individual = task.individuals.all()[0]
    print(individual.location)

    path, vcf = os.path.split(individual.location)

    task_location = '/tmp/tasks/%s/' % (task.id)
    command = 'mkdir -p %s' % (task_location)
    run(command, shell=True)
    os.chdir(task_location)

    # if not os.path.exists('%s/output/gatk'):
    # os.makedirs(directory)

    #get vcf from remote location
    worker = IWorker()
    worker.get(task.id, individual.location)
    #annotate it

    filename = vcf

    if filename.endswith('.vcf'):
        command = 'cp %s sample.vcf' % (filename)
        os.system(command)
    if filename.endswith('.gz'):
        command = 'gunzip -c -d %s > sample.vcf' % (filename)
        os.system(command)
    if filename.endswith('.zip'):
        command = 'unzip -p %s > sample.vcf' % (filename)
        os.system(command)
    if filename.endswith('.rar'):
        command = 'unrar e %s' % (filename)
        os.system(command)
        #now change filename to sample.vcf
        command = 'mv %s sample.vcf' % (filename.replace('.rar', ''))
        os.system(command)

    command = 'pynnotator -i sample.vcf'
    run(command, shell=True)

    final_file = 'ann_sample/annotation.final.vcf'
    if os.path.exists(final_file):

        command = 'zip annotation.final.vcf.zip ann_sample/annotation.final.vcf'
        run(command, shell=True)

        #send results to s3
        print('Send to S3')
        local = '/tmp/tasks/%s/annotation.final.vcf.zip' % (task.id)
        worker.put(task.id, local)
        stop = datetime.datetime.now()
        elapsed = stop - start
        individual.annotation_time = elapsed
        individual.save()
        task.status = 'annotated'
        # task.execution_time = elapsed
        task.save()

        task_location = '/tmp/tasks/%s/' % (task.id)
        command = 'rm -rf %s' % (task_location)
        run(command, shell=True)
        #add insertion task
        insert_vcf.delay(task.id)
    else:
        task.status = 'failed'
        task.retry += 1
        task.save()
        annotate_vcf.delay(task.id)

@app.task(queue="insertion")
def insert_vcf(task_id):
    task = Task.objects.get(pk=task_id)
    individual = task.individuals.all()[0]

    vcf = '/tmp/tasks/%s/annotation.final.vcf.zip' % (task.id)

    task_location = '/tmp/tasks/%s/' % (task.id)
    command = 'mkdir -p %s' % (task_location)
    run(command, shell=True)
    os.chdir(task_location)

    worker = IWorker()
    worker.get(task.id)
    task_location = '/tmp/tasks/%s/' % (task.id)
    command = 'mkdir -p %s' % (task_location)
    run(command, shell=True)
    os.chdir(task_location)

    #delete variants from individual before inserting
    Variant.objects.filter(individual=individual).delete()
    #SnpeffAnnotation.objects.filter(individual=individual).delete()
    #VEPAnnotation.objects.filter(individual=individual).delete()

    filepath = '/tmp/tasks/%s' % (task.id)

    print('Populating %s' % (individual.id))

    z = zipfile.ZipFile('%s/annotation.final.vcf.zip' % (filepath), 'r')
    data = z.open('ann_sample/annotation.final.vcf', 'r')

    start = datetime.datetime.now()
    count = 0
    variants = []
    count2 = 0

    snpeff_dict = {}
    vep_dict = {}

    for line in data:
        # print(line)
        line = line.decode("utf-8", "ignore")
        # print(line)
        # print('Hello')
        if line != '':
            if not line.startswith('#'):

                count += 1
                count2 += 1

                #bulk insert variants objects
                if count == 10000:
                    # print "Inserting %s " % (count2),
                    Variant.objects.bulk_create(variants)
                    # print ' Done!'
                    count = 0
                    variants = []

                #now parse
                variant = parse_vcf(line)
                #print(variant)
                #variant_dict['individual_id'] = individual.id
                # print 'index', variant
                # variant.snpeff.add(snpeff)
                variant_obj = Variant(
                individual=individual,
                index=variant['index'],
                pos_index=variant['pos_index'],
                chr=variant['chr'],
                pos=variant['pos'],
                variant_id=variant['variant_id'],
                ref=variant['ref'],
                alt=variant['alt'],
                qual=variant['qual'],
                filter=variant['filter'],
                info=variant['info'],
                genotype=variant['genotype'],
                genotype_col=variant['genotype_col'],
                format=variant['format'],
                read_depth=variant['read_depth'],
                gene=variant['gene'],
                mutation_type=variant['mutation_type'],
                vartype=variant['vartype'],
                genomes1k_maf=variant['genomes1k.AF'],
                dbsnp_maf=variant['dbsnp.MAF'],
                esp_maf=variant['esp6500.MAF'],
                dbsnp_build=variant['dbsnp_build'],
                sift=variant['sift'],
                sift_pred=variant['sift_pred'],
                polyphen2=variant['polyphen2'],
                polyphen2_pred=variant['polyphen2_pred'],
                # condel=variant['condel'],
                # condel_pred=variant['condel_pred'],
                cadd=variant['cadd'],
                # dann=variant['dann'],
                is_at_omim=variant['is_at_omim'],
                is_at_hgmd=variant['is_at_hgmd'],
                hgmd_entries=variant['hgmd_entries'],
                hi_index_str=variant['hi_index_str'],
                )

                # print variant['index']
                # variant_obj.save()
                #parse snpeff

                if 'snpeff' in variant:
                    #for snpeff in variant['snpeff']:
                        #snpeff = SnpeffAnnotation(
                    variant_obj.snpeff_effect=variant['snpeff'][0]['effect']
                    variant_obj.snpeff_impact=variant['snpeff'][0]['impact']
                    variant_obj.snpeff_func_class=variant['snpeff'][0]['func_class']
                    variant_obj.snpeff_codon_change=variant['snpeff'][0]['codon_change']
                    variant_obj.snpeff_aa_change=variant['snpeff'][0]['aa_change']
                    # variant_obj.snpeff_aa_len=variant['snpeff'][0]['aa_len']
                    variant_obj.snpeff_gene_name=variant['snpeff'][0]['gene_name']
                    variant_obj.snpeff_biotype=variant['snpeff'][0]['biotype']
                    variant_obj.snpeff_gene_coding=variant['snpeff'][0]['gene_coding']
                    variant_obj.snpeff_transcript_id=variant['snpeff'][0]['transcript_id']
                    variant_obj.snpeff_exon_rank=variant['snpeff'][0]['exon_rank']
                    # variant_obj.snpeff_genotype_number=variant['snpeff'][0]['genotype_number']
                    #)
                    #snpeff_dict[variant['index']] = snpeff

                #parse vep
                if 'vep' in variant:
                    #vep = VEPAnnotation(
                    variant_obj.vep_allele=variant['vep']['Allele']
                    variant_obj.vep_gene=variant['vep']['Gene']
                    variant_obj.vep_feature=variant['vep']['Feature']
                    variant_obj.vep_feature_type=variant['vep']['Feature_type']
                    variant_obj.vep_consequence=variant['vep']['Consequence']
                    variant_obj.vep_cdna_position=variant['vep']['cDNA_position']
                    variant_obj.vep_cds_position=variant['vep']['CDS_position']
                    variant_obj.vep_protein_position=variant['vep']['Protein_position']
                    variant_obj.vep_amino_acids=variant['vep']['Amino_acids']
                    variant_obj.vep_codons=variant['vep']['Codons']
                    variant_obj.vep_existing_variation=variant['vep']['Existing_variation']
                    variant_obj.vep_distance=variant['vep']['DISTANCE']
                    variant_obj.vep_strand=variant['vep']['STRAND']
                    variant_obj.vep_symbol=variant['vep']['SYMBOL']
                    variant_obj.vep_symbol_source=variant['vep']['SYMBOL_SOURCE']
                    variant_obj.vep_sift=variant['vep']['sift']
                    variant_obj.vep_polyphen=variant['vep']['polyphen2']
                    # variant_obj.vep_condel=variant['vep']['condel']
                    # variant_obj.rf_score=variant['vep']['rf_score']
                    # variant_obj.ada_score=variant['vep']['ada_score']

                    #)
                    #vep_dict[variant['index']] = vep

                if 'dbNSFP' in variant:

                    variant_obj.SIFT_score = variant['dbNSFP']['dbNSFP_SIFT_score']
                    variant_obj.SIFT_converted_rankscore = variant['dbNSFP']['dbNSFP_SIFT_converted_rankscore']
                    variant_obj.SIFT_pred = variant['dbNSFP']['dbNSFP_SIFT_pred']
                    variant_obj.Uniprot_acc_Polyphen2 = variant['dbNSFP']['dbNSFP_Uniprot_acc_Polyphen2']
                    variant_obj.Uniprot_id_Polyphen2 = variant['dbNSFP']['dbNSFP_Uniprot_id_Polyphen2']
                    variant_obj.Uniprot_aapos_Polyphen2 = variant['dbNSFP']['dbNSFP_Uniprot_aapos_Polyphen2']
                    variant_obj.Polyphen2_HDIV_score = variant['dbNSFP']['dbNSFP_Polyphen2_HDIV_score']
                    variant_obj.Polyphen2_HDIV_rankscore = variant['dbNSFP']['dbNSFP_Polyphen2_HDIV_rankscore']
                    variant_obj.Polyphen2_HDIV_pred = variant['dbNSFP']['dbNSFP_Polyphen2_HDIV_pred']
                    variant_obj.Polyphen2_HVAR_score = variant['dbNSFP']['dbNSFP_Polyphen2_HVAR_score']
                    variant_obj.Polyphen2_HVAR_rankscore = variant['dbNSFP']['dbNSFP_Polyphen2_HVAR_rankscore']
                    variant_obj.Polyphen2_HVAR_pred = variant['dbNSFP']['dbNSFP_Polyphen2_HVAR_pred']
                    variant_obj.LRT_score = variant['dbNSFP']['dbNSFP_LRT_score']
                    variant_obj.LRT_converted_rankscore = variant['dbNSFP']['dbNSFP_LRT_converted_rankscore']
                    variant_obj.LRT_pred = variant['dbNSFP']['dbNSFP_LRT_pred']
                    variant_obj.LRT_Omega = variant['dbNSFP']['dbNSFP_LRT_Omega']
                    variant_obj.MutationTaster_score = variant['dbNSFP']['dbNSFP_MutationTaster_score']
                    variant_obj.MutationTaster_converted_rankscore = variant['dbNSFP']['dbNSFP_MutationTaster_converted_rankscore']
                    variant_obj.MutationTaster_pred = variant['dbNSFP']['dbNSFP_MutationTaster_pred']
                    variant_obj.MutationTaster_model = variant['dbNSFP']['dbNSFP_MutationTaster_model']
                    variant_obj.MutationTaster_AAE = variant['dbNSFP']['dbNSFP_MutationTaster_AAE']
                    variant_obj.MutationAssessor_UniprotID = variant['dbNSFP']['dbNSFP_MutationAssessor_UniprotID']
                    variant_obj.MutationAssessor_variant = variant['dbNSFP']['dbNSFP_MutationAssessor_variant']
                    variant_obj.MutationAssessor_score = variant['dbNSFP']['dbNSFP_MutationAssessor_score']
                    variant_obj.MutationAssessor_rankscore = variant['dbNSFP']['dbNSFP_MutationAssessor_rankscore']
                    variant_obj.MutationAssessor_pred = variant['dbNSFP']['dbNSFP_MutationAssessor_pred']
                    variant_obj.FATHMM_score = variant['dbNSFP']['dbNSFP_FATHMM_score']
                    variant_obj.FATHMM_converted_rankscore = variant['dbNSFP']['dbNSFP_FATHMM_converted_rankscore']
                    variant_obj.FATHMM_pred = variant['dbNSFP']['dbNSFP_FATHMM_pred']
                    variant_obj.PROVEAN_score = variant['dbNSFP']['dbNSFP_PROVEAN_score']
                    variant_obj.PROVEAN_converted_rankscore = variant['dbNSFP']['dbNSFP_PROVEAN_converted_rankscore']
                    variant_obj.PROVEAN_pred = variant['dbNSFP']['dbNSFP_PROVEAN_pred']
                    variant_obj.Transcript_id_VEST3 = variant['dbNSFP']['dbNSFP_Transcript_id_VEST3']
                    variant_obj.Transcript_var_VEST3 = variant['dbNSFP']['dbNSFP_Transcript_var_VEST3']
                    variant_obj.VEST3_score = variant['dbNSFP']['dbNSFP_VEST3_score']
                    variant_obj.VEST3_rankscore = variant['dbNSFP']['dbNSFP_VEST3_rankscore']
                    variant_obj.MetaSVM_score = variant['dbNSFP']['dbNSFP_MetaSVM_score']
                    variant_obj.MetaSVM_rankscore = variant['dbNSFP']['dbNSFP_MetaSVM_rankscore']
                    variant_obj.MetaSVM_pred = variant['dbNSFP']['dbNSFP_MetaSVM_pred']
                    variant_obj.MetaLR_score = variant['dbNSFP']['dbNSFP_MetaLR_score']
                    variant_obj.MetaLR_rankscore = variant['dbNSFP']['dbNSFP_MetaLR_rankscore']
                    variant_obj.MetaLR_pred = variant['dbNSFP']['dbNSFP_MetaLR_pred']
                    variant_obj.Reliability_index = variant['dbNSFP']['dbNSFP_Reliability_index']
                    variant_obj.CADD_raw = variant['dbNSFP']['dbNSFP_CADD_raw']
                    variant_obj.CADD_raw_rankscore = variant['dbNSFP']['dbNSFP_CADD_raw_rankscore']
                    variant_obj.CADD_phred = variant['dbNSFP']['dbNSFP_CADD_phred']
                    variant_obj.DANN_score = variant['dbNSFP']['dbNSFP_DANN_score']
                    variant_obj.DANN_rankscore = variant['dbNSFP']['dbNSFP_DANN_rankscore']
                    variant_obj.fathmm_MKL_coding_score = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_score']
                    variant_obj.fathmm_MKL_coding_rankscore = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_rankscore']
                    variant_obj.fathmm_MKL_coding_pred = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_pred']
                    variant_obj.fathmm_MKL_coding_group = variant['dbNSFP']['dbNSFP_fathmm-MKL_coding_group']
                    variant_obj.Eigen_raw = variant['dbNSFP']['dbNSFP_Eigen-raw']
                    variant_obj.Eigen_phred = variant['dbNSFP']['dbNSFP_Eigen-phred']
                    # variant_obj.Eigen_raw_rankscore = variant['dbNSFP']['dbNSFP_Eigen-raw_rankscore']
                    variant_obj.Eigen_PC_raw = variant['dbNSFP']['dbNSFP_Eigen-PC-raw']
                    variant_obj.Eigen_PC_raw_rankscore = variant['dbNSFP']['dbNSFP_Eigen-PC-raw_rankscore']
                    variant_obj.GenoCanyon_score = variant['dbNSFP']['dbNSFP_GenoCanyon_score']
                    variant_obj.GenoCanyon_score_rankscore = variant['dbNSFP']['dbNSFP_GenoCanyon_score_rankscore']
                    variant_obj.integrated_fitCons_score = variant['dbNSFP']['dbNSFP_integrated_fitCons_score']
                    variant_obj.integrated_fitCons_rankscore = variant['dbNSFP']['dbNSFP_integrated_fitCons_rankscore']
                    variant_obj.integrated_confidence_value = variant['dbNSFP']['dbNSFP_integrated_confidence_value']
                    variant_obj.GM12878_fitCons_score = variant['dbNSFP']['dbNSFP_GM12878_fitCons_score']
                    variant_obj.GM12878_fitCons_rankscore = variant['dbNSFP']['dbNSFP_GM12878_fitCons_rankscore']
                    variant_obj.GM12878_confidence_value = variant['dbNSFP']['dbNSFP_GM12878_confidence_value']
                    variant_obj.H1_hESC_fitCons_score = variant['dbNSFP']['dbNSFP_H1-hESC_fitCons_score']
                    variant_obj.H1_hESC_fitCons_rankscore = variant['dbNSFP']['dbNSFP_H1-hESC_fitCons_rankscore']
                    variant_obj.H1_hESC_confidence_value = variant['dbNSFP']['dbNSFP_H1-hESC_confidence_value']
                    variant_obj.HUVEC_fitCons_score = variant['dbNSFP']['dbNSFP_HUVEC_fitCons_score']
                    variant_obj.HUVEC_fitCons_rankscore = variant['dbNSFP']['dbNSFP_HUVEC_fitCons_rankscore']
                    # variant_obj.HUVEC_confidence_value = variant['dbNSFP']['dbNSFP_HUVEC_confidence_value']
                    # variant_obj.GERP_NR = variant['dbNSFP']['dbNSFP_GERP++_NR']
                    # variant_obj.GERP_RS = variant['dbNSFP']['dbNSFP_GERP++_RS']
                    # variant_obj.GERP_RS_rankscore = variant['dbNSFP']['dbNSFP_GERP++_RS_rankscore']
                    # variant_obj.phyloP100way_vertebrate = variant['dbNSFP']['dbNSFP_phyloP100way_vertebrate']
                    # variant_obj.phyloP100way_vertebrate_rankscore = variant['dbNSFP']['dbNSFP_phyloP100way_vertebrate_rankscore']
                    # variant_obj.phyloP20way_mammalian = variant['dbNSFP']['dbNSFP_phyloP20way_mammalian']
                    # variant_obj.phyloP20way_mammalian_rankscore = variant['dbNSFP']['dbNSFP_phyloP20way_mammalian_rankscore']
                    # variant_obj.phastCons100way_vertebrate = variant['dbNSFP']['dbNSFP_phastCons100way_vertebrate']
                    # variant_obj.phastCons100way_vertebrate_rankscore = variant['dbNSFP']['dbNSFP_phastCons100way_vertebrate_rankscore']
                    # variant_obj.phastCons20way_mammalian = variant['dbNSFP']['dbNSFP_phastCons20way_mammalian']
                    # variant_obj.phastCons20way_mammalian_rankscore = variant['dbNSFP']['dbNSFP_phastCons20way_mammalian_rankscore']
                    # variant_obj.SiPhy_29way_pi = variant['dbNSFP']['dbNSFP_SiPhy_29way_pi']
                    # variant_obj.SiPhy_29way_logOdds = variant['dbNSFP']['dbNSFP_SiPhy_29way_logOdds']
                    # variant_obj.SiPhy_29way_logOdds_rankscore = variant['dbNSFP']['dbNSFP_SiPhy_29way_logOdds_rankscore']
                    variant_obj.clinvar_rs = variant['dbNSFP']['dbNSFP_clinvar_rs']
                    variant_obj.clinvar_clnsig = variant['dbNSFP']['dbNSFP_clinvar_clnsig']
                    variant_obj.clinvar_trait = variant['dbNSFP']['dbNSFP_clinvar_trait']
                    variant_obj.clinvar_golden_stars = variant['dbNSFP']['dbNSFP_clinvar_golden_stars']
                    variant_obj.mcap_score = variant['mcap']
                    variant_obj.mcap_rankscore = variant['dbNSFP']['dbNSFP_M-CAP_rankscore']
                    variant_obj.mcap_pred = variant['dbNSFP']['dbNSFP_M-CAP_pred']
                    variant_obj.revel_score = variant['dbNSFP']['dbNSFP_REVEL_score']

                variants.append(variant_obj)
                # print 'query', variant_obj.query
                # print(variant['chr'], variant['pos'])
                # variant_obj.save()
    Variant.objects.bulk_create(variants)

    stop = datetime.datetime.now()
    elapsed = stop - start

    individual.insertion_time = elapsed


    individual.status = 'populated'
    individual.n_lines = count2
    individual.save()

    stop = datetime.datetime.now()
    task.finished = stop
    task.status = 'populated'
    task.save()

    # message = """
    #         The individual %s was inserted to the database with success!
    #         Now you can check the variants on the link: \n
    #         http://mendelmd.org/individuals/view/%s
    #             """ % (individual.name, individual.id)

    print('Individual %s Populated!' % (individual.id))

    command = 'rm -rf %s' % (task_location)
    run(command, shell=True)

    # command = 'rm -rf %s/ann_sample' % (filepath)
    # os.system(command)



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(name='mendelmd',
      version='1.2',
      packages=find_packages(),
      scripts=['bin/mendelmd'],
      install_requires=[
          'wheel',
          'django',
          'pysam',
          'cython',
		  'wheel',
		  'Django',
		  'psycopg2-binary',
		  'django-allauth',
		  'django-debug-toolbar',
		  'django-crispy-forms',
		  'SOAPpy',
		  'Sphinx',
		  'Fabric3',
		  'pysam',
		  'pynnotator',
		  'pyvcf',
		  'celery',
		  'django-celery',
		  'django-celery-results',
		  'django-appconf',
		  'gunicorn',
		  'boto3',
		  'django-formtools',
		  'dj-database-url',
		  'django-gravatar2',
		  'pandas',
      ],
      # entry_points={  # Optional
      #         'console_scripts': [
      #             'mendelmd=mendelmd.main:main',
      #         ],
      # },
      )



# Create your tasks here
from __future__ import absolute_import, unicode_literals

from django.conf import settings  # noqa
from celery import Celery
app = Celery('mendelmd')

# Using a string here means the worker will not have to
# pickle the object when using Windows.
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)

from tasks.models import Task
from workers.models import Worker
from django.db.models import Q

from subprocess import run, check_output

from helpers.scw_wrapper import SCW
from helpers.aws_wrapper import AWS

from settings.models import Provider

@app.task(queue="master")
def check_queue():
    #check tasks and launch workers if necessary
    print('Check Queue')
    max_workers = 50
    tasks = Task.objects.filter(status='scheduled')
    workers = Worker.objects.filter(~Q(status='terminated'))
    n_tasks = len(tasks)
    n_workers = len(workers)
    print(n_tasks, n_workers)
    #if more tasks than workers, launch more workers
    if n_tasks > n_workers and n_workers < max_workers:
        n_workers_to_launch = min(n_tasks, max_workers - n_workers)
        print('Launch Workers', n_workers_to_launch)
        for i in range(0,n_workers_to_launch):
            launch_worker.delay()
    #if more workers than tasks, terminate workers
    if n_tasks < n_workers:
        print('Terminate Workers')
        terminate_workers()

@app.task(queue="master")
def launch_worker():
    #create workers
    worker = Worker()
    worker.name = 'New Worker'
    worker.status = 'new'
    worker.save()

    if settings.DEFAULT_PROVIDER == 'AWS':
        provider = Provider.objects.filter(name='AWS')[0]
        print(provider, provider.config)
        worker_result = AWS().launch(provider.config)
        worker.provider = 'AWS'
        worker.type = provider.config['instance_type']
    else:
        worker_result = SCW().launch(provider.config)
        worker.provider = 'SCW'
        worker.type = ''

    worker.ip = worker_result['ip']
    worker.worker_id = worker_result['id']
    worker.save()
    install_worker.delay(worker.id)


@app.task(queue="master")
def launch_workers(n_workers, type):
    #create workers
    workers = []
    for i in range(0, int(n_workers)):
        worker = Worker()
        worker.name = 'New Worker'
        worker.type = type
        worker.status = 'new'
        worker.save()
        workers.append(worker)

    for i, worker in enumerate(workers):
        print('Launch ', i)
        # launch new worker
        worker_result = SCW().launch(type)
        worker.ip = worker_result['ip']
        worker.worker_id = worker_result['id']
        worker.save()
        install_worker.delay(worker.id)

@app.task(queue="master")
def terminate_workers():
    idle_workers = Worker.objects.filter(status='idle')
    for worker in idle_workers:
        print('Terminate Worker')
        AWS().terminate(worker.worker_id)
        print('Terminate Worker', worker.id)
        worker.status = 'terminated'
        worker.save()

@app.task(queue="master")
def terminate_worker(worker_id):
    worker = Worker.objects.get(id=worker_id)
    
    print('Terminate Worker', worker.id)
    AWS().terminate(worker.worker_id)
    worker.status = 'terminated'
    worker.save()

@app.task(queue="master")
def install_worker(worker_id):
    worker = Worker.objects.get(id=worker_id)
    print('Install Worker', worker.id)
    if settings.DEFAULT_PROVIDER == 'AWS':
        AWS().install(worker.ip)

@app.task(queue="master")
def update_worker(worker_id):
    worker = Worker.objects.get(id=worker_id)
    print('Update Worker', worker.id)
    if settings.DEFAULT_PROVIDER == 'AWS':
        AWS().update(worker.ip)

    # command = 'rsync -avz {} root@%s:/projects/mendelmd'.format(settings.BASE_DIR, worker.ip)

@app.task(queue="master")
def check_workers():

    workers = Worker.objects.all()
    for worker in workers:

        ip = worker.ip
        # print(worker.status)
        # command = """top -b -n 10 -d.2 | grep 'Cpu' |  awk 'NR==3{ print($2)}'"""
        # command = 'top -b -n 1 | head -n 10'
        command = 'top -bcn1 -w512 | head -n 10'
        
        command = """ssh -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null ubuntu@%s %s""" % (ip,command)
        output = check_output(command, shell=True)
        # print(output.decode('utf-8'))
        text = output.decode('utf-8').splitlines()

        process_list_started = False
        for line in text:
            # print(line)
            if line.startswith('%Cpu(s)'):
                cpu_usage = line.split()[0]

            if process_list_started:
                process = line
                break

            if line.startswith('  PID USER'):
                process_list_started = True
            
        # print(process.split())
        rows = process.split()
        current_process = ' '.join(rows[10:])
        output = '{} {}'.format(cpu_usage, current_process)
        # print(output)
        worker.current_status = output
        worker.save()


from django.contrib import admin
from individuals.models import ControlGroup, UserGroup, Individual
# Register your models here.
admin.site.register(ControlGroup)
admin.site.register(UserGroup)

admin.site.register(Individual)



from django.core.management.base import BaseCommand, CommandError
from files.models import File
from settings.models import S3Credential
import boto3
import os
from django.conf import settings
import time
from subprocess import run

class Command(BaseCommand):
    help = 'Download Files'

    def handle(self, *args, **options):
        print('Hello World Import Files')
        start_time = time.time()
        print('Download Files')
        command = 'mkdir -p {}/data/files/'.format(settings.BASE_DIR)
        run(command, shell=True)
        file_list = open('%s/data/files/all_files.txt' % (settings.BASE_DIR), 'w')
        s3credentials = S3Credential.objects.all()
        for s3credential in s3credentials:
            print(s3credential.name)
            for bucket_name in s3credential.buckets.splitlines():
                session = boto3.Session(
                    aws_access_key_id=s3credential.access_key,
                    aws_secret_access_key=s3credential.secret_key
                )
                s3 = session.resource('s3')
                bucket = s3.Bucket(bucket_name)
                print(bucket)
                for key in bucket.objects.all():
                    if key.size != 0:
                        file = [str(key.last_modified), str(key.size), bucket.name, key.key]
                        file_list.writelines('%s\n' % ('\t'.join(file)))
        self.stdout.write(self.style.SUCCESS('Successfully downloaded files!'))
        elapsed_time = time.time() - start_time
        print('Importing Files Took {}'.format(elapsed_time))


from django.db import models

from django.contrib.auth.models import User
from datetime import datetime
from files.models import File
from django.urls import reverse

class Sample(models.Model):
    def get_upload_path(self, filename):
        if self.user != None:
            string = "%s/media/%s/%s/%s" % (settings.BASE_DIR, slugify(self.user.username), self.id, filename)
        else:
            string = "%s/media/public/%s/%s" % (settings.BASE_DIR, self.id, filename)
        return string

    user = models.ForeignKey(User, editable=False, null=True, on_delete=models.CASCADE)
    files = models.ManyToManyField(File, blank=True)

    # shared_with_users = models.ManyToManyField(User, editable=True, related_name="shared_with_users", blank=True)
    # shared_with_groups = models.ManyToManyField(UserGroup, editable=True, related_name="shared_with_groups", blank=True)

    name = models.CharField(max_length=600)
    is_featured = models.BooleanField(default=True)
    is_public = models.BooleanField(default=False)
    
    file = models.FileField(upload_to=get_upload_path, blank=True, help_text="File Format: VCF",max_length=600)
    file_header = models.TextField(null=True, blank=True)

    prefix = models.TextField(null=True, blank=True)


    status = models.CharField(max_length=100, blank=True, editable=False)
    n_variants = models.IntegerField(null=True, blank=True, editable=False)
    n_lines = models.IntegerField(null=True, blank=True, editable=False)

    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.creation_date:
            self.creation_date = datetime.now()
        self.modified_date = datetime.now()
        return super(Sample, self).save(*args, **kwargs)


class SampleGroup(models.Model):
    name = models.CharField(max_length=128)
    members = models.ManyToManyField(Sample,
        related_name='samplegroup_members')
    def __str__(self):
        return self.name

    
    def get_absolute_url(self):
        return reverse('samplegroup-view', kwargs={'pk': self.pk})




from django.conf.urls import patterns, include, url
from django.views.generic import DetailView, ListView, CreateView, UpdateView, DeleteView
#from filter_analysis.models import *
#from filter_analysis.views import FilterAnalysisUpdateView, FilterAnalysisDeleteView, FilterConfigUpdateView, FilterConfigDeleteView

urlpatterns = patterns('pathway_analysis.views',
    url(r'^$', 'index', name='pathway_analysis'),
    url(r'^populate/$', 'populate', name='populate_pathway'),
    url(r'^analysis/$', 'analysis', name='pathway_filter_analysis'),
    url(r'^view/(?P<pathway_id>\d+\w+)$', 'view', name='pathway_view'),
    
#    url(r'^trioanalysis/$', 'trioanalysis', name='trio_analysis'),
#    
#    
#    
#    url(r'^wizard/$', 'wizard', name='filter_wizard'),
##    url(r'^create/$', 'create', name='create_filter'),
#    url(r'^create/$', 'create', name='create_filter'),
#    url(r'^createconfig/$', 'createconfig', name='create_config'),
#    url(r'^updateanalysis/(?P<pk>\d+)', FilterAnalysisUpdateView.as_view(model=FilterAnalysis), name="analysis_update",),
#    url(r'^deleteanalysis/(?P<pk>\d+)', FilterAnalysisDeleteView.as_view(), name="analysis_delete",),
#    url(r'^updateconfig/(?P<pk>\d+)', FilterConfigUpdateView.as_view(model=FilterConfig), name="config_update",),
#    url(r'^deleteconfig/(?P<pk>\d+)', FilterConfigDeleteView.as_view(), name="config_delete",),
)



from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect


import os
# Create your views here.
from individuals.models import Individual
from django.contrib import messages
from django.conf import settings
from django.utils.text import slugify
from individuals.tasks import *

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

def index(request):
    if request.method == 'POST':
        status = request.POST['status']
        # print('status', status)
    else:
        status = ''
    if request.user.is_staff:
        if status != '':
            individuals = Individual.objects.filter(status=status).order_by('-id')
        else:
            individuals = Individual.objects.all().order_by('-id')
    elif request.user.is_authenticated:
        individuals = Individual.objects.filter(user=request.user).order_by('-id')
    else:
        individuals = Individual.objects.filter(user=None).order_by('-id')

    n_individuals = individuals.count()

    paginator = Paginator(individuals, 1000) # Show 25 contacts per page

    page = request.GET.get('page')

    try:
        individuals = paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        individuals = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        individuals = paginator.page(paginator.num_pages)


    context = {
    'n_individuals': n_individuals,
    'individuals':individuals
    }
    return render(request, 'dashboard/dashboard.html', context)

@login_required
def bulk_action(request):
    if request.method == 'POST':
        individuals = request.POST.getlist('individuals')
        print(individuals) 
        individuals = list(reversed([int(x) for x in individuals]))
        print(individuals)
        
        if request.POST['selectionField'] == "Show":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                individual.featured = True
                individual.save()
        if request.POST['selectionField'] == "Hide":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                individual.featured = False
                individual.save()
        if request.POST['selectionField'] == "Delete":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)

                individual_id = individual.id
                if individual.user:
                    username = individual.user.username
                else:
                    username = 'public'
                #delete files
                if individual.vcf_file:
                    individual.vcf_file.delete()
                # if individual.strs_file:
                #     individual.strs_file.delete()
                # if individual.cnvs_file:
                #     individual.cnvs_file.delete()
                os.system('rm -rf %s/genomes/%s/%s' % (settings.BASE_DIR, slugify(username), individual_id))

                individual.delete()
            messages.add_message(request, messages.INFO, "Individuals deleted with success!")
            #os.system('rm -rf mendelmd14/site_media/media/genomes/%s/%s' % (username, individual_id))
        if request.POST['selectionField'] == "Populate":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                PopulateVariants.delay(individual.id)
            
        if request.POST['selectionField'] == "Annotate":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                individual.status = 'new'
                individual.n_lines = 0
                individual.save()
                AnnotateVariants.delay(individual.id)
        if request.POST['selectionField'] == "Find_Medical_Conditions_and_Medicines":
            for individual_id in individuals:
                individual = get_object_or_404(Individual, pk=individual_id)
                Find_Medical_Conditions_and_Medicines.delay(individual.id)
    
    return redirect('dashboard')



from django.shortcuts import render
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.urls import reverse_lazy
from django.shortcuts import render, redirect, get_object_or_404


from .models import App
# Create your views here.
def index(request):
    
    if request.method == 'POST':
        apps = request.POST.getlist('apps')
        action = request.POST['action']
        for app_id in apps:
            if request.user.is_staff:
                app = get_object_or_404(App, pk=app_id)
            else:
                app = get_object_or_404(App, pk=app_id, user=request.user)
        
            if action == "delete":
                app.delete()

    app_list = App.objects.all()[:5]
    context = {'app_list': app_list}
    return render(request, 'mapps/index.html', context)


class AppCreate(CreateView):
    model = App
    fields = ['name', 'status', 'category', 'source', 'repository', 'type', 'config', 'main', 'install']

    def form_valid(self, form):
        form.instance.user = self.request.user
        return super().form_valid(form)


class AppUpdate(UpdateView):
    model = App
    fields = ['name', 'status', 'category', 'source', 'repository', 'type', 'config', 'main', 'install']

    def form_valid(self, form):
        form.instance.user = self.request.user
        return super().form_valid(form)
